@@ -3,10 +3,59 @@
 ;; Author: Healfdene Goguen
 ;; Maintainer: Pierre Courtieu <courtieu@lri.fr>
 
-;; coq.el,v 7.0 2002/08/29 09:12:08 da Exp 
+;; coq.el,v 8.0 2004/04/17 23:39:58 da Exp 
 
 (require 'proof)
+(require 'holes) ; in generic directory
+;; coq-syntaxe is required below
+;; ----- coq-shell configuration options
+
+(defcustom coq-prog-name "coqtop -emacs"
+  "*Name of program to run as Coq."
+  :type 'string
+  :group 'coq)
+
+;; Command to initialize the Coq Proof Assistant
+
+(defcustom coq-default-undo-limit 100
+  "Maximum number of Undo's possible when doing a proof."
+  :type 'number
+  :group 'coq)
+
+(defconst coq-shell-init-cmd 
+  (format "Set Undo %s. " coq-default-undo-limit))
+
+;; da 15/02/03: moved setting of coq-version-is-vX to coq-syntax to
+;; fix compilation
+
 (require 'coq-syntax)
+(require 'coq-indent)
+
+;; Command to reset the Coq Proof Assistant
+;; Pierre: added Impl... because of a bug of Coq until V6.3
+;; (included). The bug is already fixed in the next version (V7). So
+;; we will backtrack this someday.
+(defconst coq-shell-restart-cmd 
+  "Reset Initial.\n Implicit Arguments Off. ")
+
+
+(defvar coq-shell-prompt-pattern (concat "^" proof-id " < ")
+  "*The prompt pattern for the inferior shell running coq.")
+
+;; FIXME da: this was disabled (set to nil) -- why?
+(defvar coq-shell-cd "Cd \"%s\". "
+  "*Command of the inferior process to change the directory.") 
+
+(defvar coq-shell-abort-goal-regexp "Current goal aborted"
+  "*Regexp indicating that the current goal has been abandoned.")
+
+(defvar coq-shell-proof-completed-regexp "Subtree proved!"
+  "*Regular expression indicating that the proof has been completed.")
+
+(defvar coq-goal-regexp
+  "\\(============================\\)\\|\\(subgoal [0-9]+ is:\\)\n")
+
+
 
 ;; Configuration
 
@@ -27,11 +76,6 @@
 (defconst coq-interrupt-regexp "User Interrupt."
   "Regexp corresponding to an interrupt")
 
-(defcustom coq-default-undo-limit 100
-  "Maximum number of Undo's possible when doing a proof."
-  :type 'number
-  :group 'coq)
-
 ;; ----- web documentation
 
 (defcustom coq-www-home-page "http://coq.inria.fr/"
@@ -39,72 +83,59 @@
   :type 'string
   :group 'coq)
 
-;; ----- coq specific menu
 
-(defpgdefault menu-entries
-  '(["Print..." coq-Print t]
-    ["Check..." coq-Check t]
-    ["Hints" coq-PrintHint t]
-    ["Intros..." coq-Intros t]
-    ["Show ith goal..." coq-Show t]
-    ["Apply"  coq-Apply t]
-    ["Search isos/pattern..." coq-SearchIsos t]
-    ["Begin Section..." coq-begin-Section t]
-    ["End Section" coq-end-Section t]
-    ["Compile" coq-Compile t]))
+;; ----- coq specific menu is defined in coq-abbrev.el
+
+(cond
+ (coq-version-is-V8 (require 'coq-abbrev))
+ (t (require 'coq-abbrev-V7))
+ )
 
 
-;; ----- coq-shell configuration options
-
-(defcustom coq-prog-name "coqtop -emacs"
-  "*Name of program to run as Coq."
-  :type 'string
-  :group 'coq)
-
-;; Command to initialize the Coq Proof Assistant
-(defconst coq-shell-init-cmd 
-  (format "Set Undo %s. " coq-default-undo-limit))
 
 
-;;Pierre: we will have both versions V6 and V7 during a while
-;;        the test with "coqtop -v" can be skipped if the variable 
-;;        coq-version-is-V7 is already set (usefull for people 
-;;        dealing with several version of coq)
-(if (boundp 'coq-version-is-V7) () ; if this variable is bound, do nothing 
-  (setq coq-version-is-V7          ; else test with "coqtop -v"
-	(if (string-match "version 7" (shell-command-to-string (concat coq-prog-name " -v"))) 
-	    (progn (message "coq is V7") t)
-	  (progn (message "coq is not V7") nil))))
+(defun coq-insert-section (s)
+  (interactive  "sSection name: ")
+  (let ((p (point)))
+    (insert "Section " s ".\n#\nEnd " s ".")
+    (replace-string-by-holes-backward-move-point 1 empty-hole-string)
+    (goto-char p)
+    (set-point-next-hole-destroy))
+)
 
-;; Command to reset the Coq Proof Assistant
-;; Pierre: added Impl... because of a bug of Coq until V6.3
-;; (included). The bug is already fixed in the next version (V7). So
-;; we will backtrack this someday.
-(defconst coq-shell-restart-cmd 
-  "Reset Initial.\n Implicit Arguments Off. ")
+(defconst module-kinds-table 
+  '(("Module" 1) ("Module Type" 2) ("Declare Module" 3))
+  "Enumerates the different kinds of modules.")
 
-(defvar coq-shell-prompt-pattern (concat "^" proof-id " < ")
-  "*The prompt pattern for the inferior shell running coq.")
+(defconst modtype-kinds-table
+  '(("" 1) (":" 2) ("<:" 3))
+  "Enumerates the different kinds of type information for modules.")
 
-;; FIXME da: this was disabled (set to nil) -- why?
-(defvar coq-shell-cd "Cd \"%s\". "
-  "*Command of the inferior process to change the directory.") 
-
-(defvar coq-shell-abort-goal-regexp "Current goal aborted"
-  "*Regexp indicating that the current goal has been abandoned.")
-
-(defvar coq-shell-proof-completed-regexp "Subtree proved!"
-  "*Regular expression indicating that the proof has been completed.")
-
-(defvar coq-goal-regexp
-  "\\(============================\\)\\|\\(subgoal [0-9]+ is:\\)\n")
+(defun coq-insert-module ()
+  (interactive)
+  (let* ((mods (completing-read "kind of module (tab to see list): " module-kinds-table))
+         (s (read-string  "Module name: "))
+         (typkind (completing-read "kind of type (optional, tab to see list): " modtype-kinds-table))
+         (p (point)))
+    (if (string-equal typkind "")
+        (progn
+          (insert mods " " s ".\n#\nEnd " s ".")
+          (replace-string-by-holes-backward-move-point 1 empty-hole-string)
+          (goto-char p))
+      (insert mods " " s " " typkind " #.\n#\nEnd " s ".")
+      (replace-string-by-holes-backward-move-point 2 empty-hole-string)
+      (goto-char p)
+      (set-point-next-hole-destroy))
+    )
+  )
+;  (completing-read "Section name: " )
 
 ;; ----- outline
 
 (defvar coq-outline-regexp
   (concat "(\\*\\|" (proof-ids-to-regexp 
 	   '(
-"Tactic" "Axiom" "Parameter" "Parameters" "Variable" "Variables" "Syntax" "Grammar" "Syntactic" "Load" "Require" "Hint" "Hints" "Hypothesis" "Correctness" "Section" "Chapter" "Goal" "Lemma" "Theorem" "Fact" "Remark" "Record" "Inductive" "Mutual" "Definition" "Fixpoint" "Save" "Qed" "Defined" "End" "Coercion"))))
+"Tactic" "Axiom" "Parameter" "Parameters" "Variable" "Variables" "Syntax" "Grammar" "Syntactic" "Load" "Require" "Import" "Hint" "Hints" "Hypothesis" "Correctness" "Module" "Section" "Chapter" "Goal" "Lemma" "Theorem" "Fact" "Remark" "Record" "Inductive" "Mutual" "Definition" "Fixpoint" "Save" "Qed" "Defined" "End" "Coercion"))))
 
 (defvar coq-outline-heading-end-regexp "\\*\)\n\\|\\.\n")
 
@@ -131,6 +162,17 @@
   (proof-ids-to-regexp coq-retractable-instruct))
 (defvar coq-non-retractable-instruct-regexp
   (proof-ids-to-regexp coq-non-retractable-instruct))
+
+(defvar coq-keywords-section
+  (cond 
+   (coq-version-is-V74 '("Section" "Module\\-+Type" "Declare\\s-+Module" "Module"))
+   (t '("Section"))))
+
+(defvar coq-section-regexp 
+  (concat "\\(" (proof-ids-to-regexp coq-keywords-section) "\\)")
+;  "\\(\\<Section\\>\\|\\<Module\\>\\-+\\<Type\\>\\|\\<Module\\>\\)"
+)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;   Derived modes - they're here 'cos they define keymaps 'n stuff ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -174,148 +216,126 @@
   (proof-ids-to-regexp (append coq-keywords-decl coq-keywords-defn))
   "Declaration and definition regexp.")
 
-;; FIXME da: this one function breaks the nice configuration of Proof General:
-;; would like to have proof-goal-regexp instead.
-;; Unfortunately Coq allows "Definition" and friends to perhaps have a goal, 
-;; so it appears more difficult than just a proof-goal-regexp setting.
-;; Future improvement may simply to be allow a function value for
-;; proof-goal-regexp.
-
-(defun coq-goal-command-p (str)
-  "Decide whether argument is a goal or not"
-  (and (proof-string-match coq-goal-command-regexp str)
-       (not (proof-string-match "Definition.*:=" str))
-		 (not (proof-string-match "Recursive Definition" str))
-       ;; da: 3.4 test: do not exclude Lemma: we want goal-command-p to
-       ;; succeed for nested goals too now.
-       ;; (should we also exclude Definition?)
-       (not (proof-string-match "Lemma.*:=" str))))
-;; ))
-
-
 ; Pierre: This is a try, for use in find-and-forget. It seems to work but 
 ; is it correct with respect to the asynchronous communication between Coq 
 ; and emacs?  
 ; DA: should be okay, communication is not as asynchronous as you think
 (defun coq-proof-mode-p ()
   "Allows to know if we are currentlly in proof mode. 
-Look at the last line of the *coq* buffer to see if the prompt 
-is the toplevel \"Coq <\". Returns nil if yes. 
-This assumes that no \"Resume\" command has been used."
+Look at the last line of the *coq* buffer to see if the prompt is the
+toplevel \"Coq <\". Returns nil if yes. This assumes that no
+\"Resume\" or \"Suspend\" command has been used."
   (not (string-match "^Coq < " proof-shell-last-prompt)))
 
 
-;; TODO : add the stuff to handle the "Correctness" command 
-
 ;; Pierre: May 29 2002 added a "Back n. " command in order to
 ;; synchronize more accurately.
  
 ;; DA: rewrote to combine behaviour with count-undos, to work with
 ;; nested proofs.
 
+
+(defun coq-is-comment-or-proverprocp (span) 
+  (memq (span-property span 'type) '(comment proverproc)))
+(defun coq-is-goalsave-p (span) (eq (span-property span 'type) 'goalsave))
+(defun coq-is-module-equal-p (str) ;;cannot appear vith coq < 7.4
+  (and (proof-string-match "\\`\\(Declare\\s-\\)?\\s-*\\<Module\\>.*:=" str)
+       (not (proof-string-match "\\<with\\>" str))))
+(defun coq-is-def-p (str) 
+  (proof-string-match (concat "\\`Definition\\s-+\\(" proof-id "\\)\\s-*") str))
+(defun coq-is-decl-defn-p (str)
+  (proof-string-match 
+   (concat "\\`\\(" coq-keywords-decl-defn-regexp "\\)\\s-*\\(" 
+           proof-id "\\)\\s-*[\\[,:.]") str))
+
+(defun coq-state-preserving-command-p (str)
+  (proof-string-match (concat "\\`\\(" coq-state-preserving-commands-regexp "\\)") str))
+
+(defun coq-state-preserving-tactic-p (str)
+  (proof-string-match (concat "\\`\\(" coq-state-preserving-tactics-regexp "\\)") str))
+
+(defun coq-state-changing-command-p (str)
+  (proof-string-match (concat "\\`\\(" coq-state-changing-commands-regexp "\\)") str))
+
+; if no second id --> name of the module/section is (match-string 2 str)
+; otherwise it is (match-string 5 str) 
+; to know if there is a second id: (match-string 2 str)="Type" ?
+(defun coq-section-or-module-start-p (str)
+  (proof-string-match 
+   (concat "\\`" coq-section-regexp 
+           "\\s-+\\(" proof-id "\\)\\(\\s-*\\(" proof-id "\\)\\)?") str)) 
+
+;;(span-at (point) 'type)
+;;(span-property (span-at (point) 'type) 'cmd)
+
 (defun coq-find-and-forget (span)
   (let (str ans (naborts 0) (nbacks 0) (nundos 0))
     (while (and span (not ans))
       (setq str (span-property span 'cmd))
       (cond
-       ((eq (span-property span 'type) 'comment))
-       
+       ((coq-is-comment-or-proverprocp span)) ; do nothing
+
+       ;; Module <Type> T ... :=... . inside proof ->  like Definition...:=...  
+       ;; (should actually be disallowed in coq)
+       ; Should go in last cond, but I have a problem trying to avoid next cond to match.
+       ((and (coq-proof-mode-p) (coq-is-module-equal-p str)) (incf nbacks))
+
        ;; FIXME: combine with coq-keywords-decl-defn-regexp case below?
        ;; [ Maybe not: Section is being treated as a _goal_ command
        ;;   now, so this test has to appear before the goalsave ]
-       ((proof-string-match 
-         (concat "Section\\s-+\\(" proof-id "\\)\\s-*") str)
-        (unless (eq (span-property span 'type) 'goalsave)
-          ;; If we're resetting to beginning of a section from
-          ;; inside, need to fix the nesting depth.
-          ;; FIXME: this is not good enough: the scanning loop
-          ;; exits immediately but Reset has implicit Aborts
-          ;; which are not being counted here.  Really
-          ;; we need to set the "master reset" command which
-          ;; subsumes the others, but still count the depth.
-          (decf proof-nesting-depth))
-        (setq ans (format coq-forget-id-command (match-string 2 str))))
-       
-       ((eq (span-property span 'type) 'goalsave)
-        ;; Note da 6.10.99: in Lego and Isabelle, it's trivial to forget an
-        ;; unnamed theorem.  Coq really does use the identifier
-        ;; "Unnamed_thm", though!  So we don't need this test:
-        ;;(unless (eq (span-property span 'name) proof-unnamed-theorem-name)
+       ((coq-section-or-module-start-p str)
+        (if (equal (match-string 2 str) "Type") ;Module Type id: take the third word
+            (setq ans (format coq-forget-id-command (match-string 5 str)))
+          (setq ans (format coq-forget-id-command (match-string 2 str))))
+        ;; If we're resetting to beginning of a section from inside, need to fix the
+        ;; nesting depth.  FIXME: this is not good enough: the scanning loop exits
+        ;; immediately but Reset has implicit Aborts which are not being counted
+        ;; here.  Really we need to set the "master reset" command which subsumes the
+        ;; others, but still count the depth.
+        (unless (coq-is-goalsave-p span) (decf proof-nesting-depth)))
 
-        ;; da: try using just Back since "Reset" causes loss of proof
-        ;; state.
-        ;; (format coq-forget-id-command (span-property span 'name)))
-        (if (span-property span 'nestedundos) 
-            (setq nbacks (+ 1 nbacks (span-property span 'nestedundos)))))
+       ((coq-is-goalsave-p span)
+        ;; da: try using just Back since "Reset" causes loss of proof state.
+        (if (span-property span 'nestedundos)
+            ;; Pierre: more robust when some unknown commands are not well backtracked
+            (if (and (= (span-property span 'nestedundos) 0) (not (coq-proof-mode-p))) 
+                (setq ans (format coq-forget-id-command (span-property span 'name)))
+              (setq nbacks (+ 1 nbacks (span-property span 'nestedundos))))))
        
        ;; Unsaved goal commands: each time we hit one of these
        ;; we need to issue Abort to drop the proof state.
-       ((coq-goal-command-p str)
-        (incf naborts))
-       
-       ;; If we are already outside a proof, issue a Reset.
-       ;; [ improvement would be to see if the undoing
-       ;;   will take us outside a proof, and use the first
-       ;;   Reset found if so: but this is tricky to co-ordinate
-       ;;   with the number of Backs, perhaps? ]
-       ((and 
-         (not (coq-proof-mode-p));; (eq proof-nesting-depth 0)
-         (proof-string-match 
-          (concat "\\`\\(" coq-keywords-decl-defn-regexp "\\)\\s-*\\(" 
-                  proof-id 
-                  ;; Section .. End Section should be atomic!
-                  "\\)\\s-*[\\[,:.]") str))
-        (setq ans (format coq-forget-id-command (match-string 2 str))))
+       ((coq-goal-command-p str) (incf naborts))
 
-       ;; FIXME: combine with coq-keywords-decl-defn-regexp case above?
-       ;; If it's not a goal but it contains "Definition" then it's a
-       ;; declaration  [ da: is this not covered by above case??? ]
-       ((and (not (coq-proof-mode-p));; (eq proof-nesting-depth 0)
-             (proof-string-match
-              (concat "Definition\\s-+\\(" proof-id "\\)\\s-*") str))
+       ;; If we are already outside a proof, issue a Reset.  [ improvement would be
+       ;; to see if the undoing will take us outside a proof, and use the first Reset
+       ;; found if so: but this is tricky to co-ordinate with the number of Backs,
+       ;; perhaps? ]
+       ((and (not (coq-proof-mode-p));; (eq proof-nesting-depth 0) 
+             (coq-is-decl-defn-p str))
         (setq ans (format coq-forget-id-command (match-string 2 str))))
 		 
        ;; Outside a proof: cannot be a tactic, if unknown: do back 
        ;; (we may decide otherwise, it is false anyhow, use elisp 
        ;; vars instead for the perfect thing).
-       ((and (not (coq-proof-mode-p))
-             (not (proof-string-match 
-                   (concat "\\`\\(" 
-                           coq-state-preserving-commands-regexp
-                           "\\)")
-                   str)))
+       ((and (not (coq-proof-mode-p)) (not (coq-state-preserving-command-p str)))
         (incf nbacks))
 
-       ;; inside a proof: if known command then back or nothing (depending 
-       ;;                 on the command), 
-       ;; if known "need not undo tactic" then nothing
-       ;; otherwise : undo (unknown tactics are considered needing undo, 
-       ;;                    which is ok at 99%, use elisp vars for the 
-       ;;                    1% remaining).
+       ;; inside a proof: if known command then back if necessary, nothing otherwise,
+       ;; if known "need not undo tactic" then nothing; otherwise : undo (unknown
+       ;; tactics are considered needing undo, use elisp vars for the 1% remaining).
        ;; no undo if abort
        ((coq-proof-mode-p)
-        (cond
-         ((proof-string-match 
-           (concat "\\`\\(" coq-state-changing-commands-regexp "\\)")
-           str)
-          (incf nbacks))
+        (cond 
+         ((coq-state-changing-command-p str) (incf nbacks))
          ((and (eq 0 naborts)
-               (not (proof-string-match 
-                     (concat "\\`\\(" coq-state-preserving-commands-regexp "\\)")
-                     str))
-               (not (proof-string-match 
-                     (concat "\\`\\(" coq-state-preserving-tactics-regexp "\\)")
-                     str)))
-          (incf nundos))
-         ))
-       )
+               (not (coq-state-preserving-command-p str))
+               (not (coq-state-preserving-tactic-p str)))
+          (incf nundos)))))
       (setq span (next-span span 'type)))
-    
-    ;; Now adjust proof-nesting depth according to the
-    ;; number of proofs we've passed out of.
-    ;; FIXME: really this adjustment should be on the
-    ;; successful completion of the Abort commands, as
-    ;; a callback.
+
+    ;; Now adjust proof-nesting depth according to the number of proofs we've passed
+    ;; out of.  FIXME: really this adjustment should be on the successful completion
+    ;; of the Abort commands, as a callback.
     (setq proof-nesting-depth (- proof-nesting-depth naborts))
 
     (setq ans
@@ -332,9 +352,12 @@ This assumes that no \"Resume\" command has been used."
            (if (> nundos 0) 
                (concat "Undo " (int-to-string nundos) ". "))))
 
-    (if (null ans) 
-        proof-no-command;; FIXME: this is an error really (assert nil)
-      ans)))
+    (if (null ans) proof-no-command;; FIXME: this is an error really (assert nil); 
+      (if (string-equal ans "") proof-no-command ; not here because if
+						 ; we backtrack a state preserving command, we must do
+						 ; *nothing*, not even a CR (in v74, no prompt is returned
+						 ; with "\n")
+      ans))))
   
 
 (defvar coq-current-goal 1
@@ -377,7 +400,7 @@ This is specific to coq-mode."
 (defun coq-guess-or-ask-for-string (s)
   (let ((guess
 			(if (region-exists-p) 
-				 (buffer-substring (region-beginning) (region-end))
+				 (buffer-substring-no-properties (region-beginning) (region-end))
 				 (symbol-near-point))))
 	 (read-string 
 	  (if guess (concat s " (" guess "):")(concat s ":"))
@@ -430,9 +453,9 @@ This is specific to coq-mode."
 		 (if (char-equal (char-after (point)) ?E)
 		     (setq count (1+ count))
 		   (setq count (1- count))))
-	       (buffer-string
-		(progn (beginning-of-line) (forward-word 1) (point)) 
-		(progn (end-of-line) (point)))))))
+	       (buffer-substring-no-properties
+          (progn (beginning-of-line) (forward-word 1) (point)) 
+          (progn (end-of-line) (point)))))))
       (insert (concat "End" section)))))
 
 (defun coq-Compile ()
@@ -444,14 +467,25 @@ This is specific to coq-mode."
 
 (proof-defshortcut coq-Intros        "Intros "  [(control ?i)])
 (proof-defshortcut coq-Apply         "Apply "   [(control ?a)])
-(proof-defshortcut coq-begin-Section "Section " [(control ?s)])
+;(proof-defshortcut coq-begin-Section "Section " [(control ?s)])
 
+(define-key coq-keymap [(control ?s)] 'coq-insert-section)
+(define-key coq-keymap [(control ?m)] 'coq-insert-module)
 (define-key coq-keymap [(control ?e)] 'coq-end-Section)
-(define-key coq-keymap [(control ?m)] 'coq-Compile)
 (define-key coq-keymap [(control ?o)] 'coq-SearchIsos)
 (define-key coq-keymap [(control ?p)] 'coq-Print)
 (define-key coq-keymap [(control ?c)] 'coq-Check)
 (define-key coq-keymap [(control ?h)] 'coq-PrintHint)
+;; da: I've moved this three buffer layout into the main code now,
+;; making it default for three bufer mode.  The function 
+;; `proof-layout-windows' lays out according to current display
+;; mode: you can use this (C-c C-l) to do what binding below did.
+;;(define-key coq-keymap [(control f3)] 'coq-three-b)
+
+; This is arguable, but completion with a three key shortcut is bad, 
+; and the feault meta-/ is bad on some keyboards (especially french ones)
+(define-key global-map [(control backspace)] 'expand-abbrev)
+
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -525,6 +559,7 @@ This is specific to coq-mode."
 	   " -emacs"))
       coq-prog-name)))
 
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;   Coq shell startup and exit hooks                               ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -543,8 +578,10 @@ This is specific to coq-mode."
 (defun coq-mode-config ()
 
   (setq proof-terminal-char ?\.)
-  (setq proof-script-command-end-regexp   
-	(if coq-version-is-V7 "[.]\\([ \t\n\r]\\)\\|[.]\\'" "[.]"))
+  (setq proof-script-command-end-regexp
+        (if coq-version-is-V7 
+            "\\(?:\\w\\|\\s-\\|\\s)\\|\\(?:\\.\\.\\)+\\)\\.\\(\\s-\\|\\'\\)"
+          "[.]"))
   (setq proof-script-comment-start "(*")
   (setq proof-script-comment-end "*)")
   (setq proof-unnamed-theorem-name "Unnamed_thm") ; Coq's default name
@@ -574,24 +611,30 @@ This is specific to coq-mode."
 	proof-state-preserving-p 'coq-state-preserving-p)
 
   (setq proof-save-command-regexp coq-save-command-regexp
+        proof-really-save-command-p 'coq-save-command-p ;pierre:deals with Proof <term>.
 	proof-save-with-hole-regexp coq-save-with-hole-regexp
 	proof-goal-with-hole-regexp coq-goal-with-hole-regexp
-	proof-nested-undo-regexp coq-state-changing-commands-regexp)
+	proof-nested-undo-regexp coq-state-changing-commands-regexp
+  proof-script-imenu-generic-expression coq-generic-expression
+  )
   
   (setq	
-   proof-indent-close-offset -1
-   proof-indent-any-regexp
-   (proof-regexp-alt (proof-ids-to-regexp
-		      (append (list "Cases" "end") coq-keywords)) "\\s(" "\\s)")
-   proof-indent-enclose-regexp "|"
-   proof-indent-open-regexp
-   (proof-regexp-alt (proof-ids-to-regexp '("Cases")) "\\s(")
-   proof-indent-close-regexp
-   (proof-regexp-alt (proof-ids-to-regexp '("end")) "\\s)"))
+;indentation is implemented in coq-indent.el
+;   proof-indent-enclose-offset  (- proof-indent)
+;   proof-indent-enclose-offset 0
+;   proof-indent-close-offset 0
+;   proof-indent-open-offset 0
+   proof-indent-any-regexp      coq-indent-any-regexp
+;   proof-indent-inner-regexp    coq-indent-inner-regexp
+;   proof-indent-enclose-regexp  coq-indent-enclose-regexp
+   proof-indent-open-regexp     coq-indent-open-regexp
+   proof-indent-close-regexp    coq-indent-close-regexp
+)
+
 
   ;; span menu 
-  (setq
-   proof-script-span-context-menu-extensions 'coq-create-span-menu)
+  (setq proof-script-span-context-menu-extensions 'coq-create-span-menu)
+
 
   ;; (setq proof-auto-multiple-files t)	; until Coq has real support
   ;; da: Experimental support for multiple files based on discussions
@@ -624,6 +667,8 @@ This is specific to coq-mode."
 		  (message "%s%s.v" dir modname)
 		  (format "%s%s.v" dir modname)))))
 
+  ;; FIXME da: coq-proof-shell-inform-file-retracted-cmd, etc, need
+  ;; defvars somewhere to avoid compiler warnings (or TBD: easy-config)
   (setq coq-proof-shell-inform-file-retracted-cmd
 	;; da: This is a HORRIBLE fragile hack!  Instead of issuing a
 	;; command to the prover we have to run a "coqdep" command to
@@ -680,8 +725,6 @@ This is specific to coq-mode."
 
   (setq font-lock-keywords coq-font-lock-keywords-1)
 
-  (setq proof-font-lock-zap-commas t)	; enable the painful hack
-  
   (proof-config-done)
 
 ;; outline
@@ -783,17 +826,16 @@ This is specific to coq-mode."
   "Whether to automatically compile vos and track dependencies."
   :type 'boolean
   :eval (if coq-auto-compile-vos
-	    (setq proof-shell-inform-file-processed-cmd 
-		  coq-proof-shell-inform-file-processed-cmd
-		  proof-shell-process-file
-		  coq-proof-shell-process-file
-		  proof-shell-inform-file-retracted-cmd
-		  coq-proof-shell-inform-file-retracted-cmd)
-	  (setq proof-shell-inform-file-processed-cmd nil
-		proof-shell-process-file nil
-		proof-shell-inform-file-retracted-cmd nil)))
+            (setq proof-shell-inform-file-processed-cmd 
+                  coq-proof-shell-inform-file-processed-cmd
+                  proof-shell-process-file
+                  coq-proof-shell-process-file
+                  proof-shell-inform-file-retracted-cmd
+                  coq-proof-shell-inform-file-retracted-cmd)
+          (setq proof-shell-inform-file-processed-cmd nil
+                proof-shell-process-file nil
+                proof-shell-inform-file-retracted-cmd nil)))
 
-(provide 'coq)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -804,7 +846,7 @@ This is specific to coq-mode."
 (defun coq-fake-constant-markup ()
   "Markup constants in Coq goal output by matching on regexps.
 This is a horrible and approximate way of doing subterm markup.
-(Code used to be in Coq, but got lost between versions 5 and 7).
+\(Code used to be in Coq, but got lost between versions 5 and 7).
 This is a hook setting for `pg-after-fontify-output-hook' to
 enable identifiers to be highlighted and allow useful 
 mouse activation."
@@ -818,18 +860,49 @@ mouse activation."
 ;; Context-senstive in-span menu additions 
 ;;
 
+;; da: message to Pierre: I just put these in as examples,
+;; maybe you can suggest some better commands to use on
+;; `thm'.  (Check maybe not much use since appears before
+;; in the buffer anyway)
+
 (defun coq-create-span-menu (span idiom name)
   (if (string-equal idiom "proof")
       (let ((thm (span-property span 'name)))
         (list (vector
+               "Check"
+               `(proof-shell-invisible-command 
+                 ,(format "Check %s." thm)))
+              (vector
                "Print Proof"
                `(proof-shell-invisible-command 
-                 ,(format "Print Proof %s." thm)))))))
+                 ,(format "Print Proof %s." thm))))))
+  (if (string-equal idiom "proof")
+      (let ((thm (span-property span 'name)))
+        (list (vector
+               "Check"
+               `(proof-shell-invisible-command 
+                 ,(format "Check %s." thm))))))
+  )
 
 
+;load the default coq abbrev file if no coq abbrev file is already read
+
+(if (and (boundp 'coq-mode-abbrev-table)
+         (not (equal coq-mode-abbrev-table (make-abbrev-table))))
+    (message "An abbrev table exists for coq, no default loaded")
+  (progn
+    (quietly-read-abbrev-file "coq-abbrev.el")
+    (message "coq default abbreviations loaded"))
+  )
+
+
+
+(provide 'coq)
+
 
 ;;;   Local Variables: ***
 ;;;   tab-width:2 ***
+;;;   fill-column: 85 ***
 ;;;   indent-tabs-mode:nil ***
 ;;;   End: ***
 
