@@ -4,12 +4,15 @@
 ;; Author:     David Aspinall and others
 ;; License:    GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; pg-user.el,v 7.0 2002/08/29 09:12:45 da Exp
+;; pg-user.el,v 8.0 2004/04/17 23:39:59 da Exp
 ;;
 ;;
 
 (require 'proof-config)			; for proof-follow-mode
 
+
+
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; first a couple of helper functions 
@@ -24,10 +27,14 @@
 	,@body)))
 
 (defun proof-maybe-follow-locked-end ()
-  "Maybe point to the make sure the locked region is displayed."
-  (if (eq proof-follow-mode 'follow)
-    (proof-goto-end-of-queue-or-locked-if-not-visible)))
-
+  "Maybe move point to make sure the locked region is displayed."
+  (cond
+   ((eq proof-follow-mode 'follow)
+    (proof-goto-end-of-queue-or-locked-if-not-visible))
+   ((eq proof-follow-mode 'followdown)
+    (if (> (proof-queue-or-locked-end) (point))
+	(goto-char (proof-queue-or-locked-end))))))
+	
 
 ;;
 ;; Doing commands
@@ -457,16 +464,22 @@ Start up the proof assistant if necessary."
 ;; command.
 ;;
 
-(proof-define-assistant-command proof-prf  
+(proof-define-assistant-command proof-prf
   "Show the current proof state."
-  proof-showproof-command)
+  proof-showproof-command
+  (progn 
+    (pg-goals-buffers-hint)
+    proof-showproof-command))
+
 (proof-define-assistant-command proof-ctxt 
   "Show the current context."
   proof-context-command)
+
 (proof-define-assistant-command proof-help 
   "Show a help or information message from the proof assistant.
 Typically, a list of syntax of commands available."
   proof-info-command)
+
 (proof-define-assistant-command proof-cd
   "Change directory to the default directory for the current buffer."
   proof-shell-cd-cmd
@@ -548,7 +561,7 @@ Make sure the modeline is updated to display new value for electric terminator."
 
 (defun proof-electric-term-incomment-fn ()
   "Used as argument to proof-assert-until-point."
-  ;; CAREFUL: (1) dynamic scoping here 
+  ;; CAREFUL: (1) dynamic scoping here  (incomment, ins, mrk)
   ;;          (2) needs this name to be recognized in 
   ;;		  proof-assert-until-point
   (setq incomment t)
@@ -705,10 +718,11 @@ last use time, to discourage saving these into the users database."
      (goto-char (span-end span))
      (skip-chars-backward " \t\n")
      (point)))
-  (own-clipboard (car kill-ring)))
+  (if (fboundp 'own-clipboard)		;; XEmacs function
+      (own-clipboard (car kill-ring))))
 
 ;; 3.3: these functions are experimental, in that they haven't
-;; been rigorously tested.  They don't work well in FSF Emacs.
+;; been rigorously tested.  FIXME: they don't work well in GNU Emacs.
 
 (defun pg-numth-span-higher-or-lower (span num &optional noerr)
   "Find NUM'th span after/before SPAN.  NUM is positive for after."
@@ -741,6 +755,7 @@ last use time, to discourage saving these into the users database."
   (or (span-property span 'controlspan)
       span))
 
+;; Really a drag-and-drop interface for this would be nice.
 (defun pg-move-span-contents (span num)
   "Move SPAN up/downwards in the buffer, past NUM spans.
 If NUM is negative, move upwards.  Return new span."
@@ -797,7 +812,7 @@ If NUM is negative, move upwards.  Return new span."
 
 (defun pg-fixup-children-span (span)
   (if (span-property span 'controlspan)
-      ;; WARNING: dynamic binding
+      ;; WARNING: dynamic binding for new-span
       (progn
 	(set-span-property span 'controlspan new-span)
 	(list span))))
@@ -816,6 +831,30 @@ If NUM is negative, move upwards.  Return new span."
   (interactive "p")
   (pg-move-region-down (- num)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; Dragging regions around 
+;;
+;; FIXME: unfinished
+
+;(defvar pg-drag-region-point nil
+;  "Temporary store of point being dragged.")
+
+;(defun pg-mouse-drag-move-region-function (event click-count dragp)
+;  (save-excursion
+;    (let* ((span (span-at (mouse-set-point event) 'type)))
+;      (if span 
+;	  (if pg-drag-region-point
+;	      ;; Move the region at point to region here.
+	      
+
+	  
+;(defun pg-mouse-drag-up-move-region-function (event click-count)
+;  (setq pg-drag-region-point nil))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 ;; FIXME: not working right yet, sigh...
 (defun proof-forward-command (&optional num)
   "Move forward to the start of the next proof region."
@@ -973,7 +1012,121 @@ If NUM is negative, move upwards.  Return new span."
 ;	     )))))
 
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; Message buffer hints  (added in PG 3.5)
+;;
 
+(defun pg-goals-buffers-hint ()
+  (pg-hint "Use \\[proof-display-some-buffers] to rotate output buffers; \\[pg-response-clear-displays] to clear response & trace."))
+
+(defun pg-response-buffers-hint (&optional nextbuf)
+  (pg-hint
+   (format
+    "\\[proof-prf] displays goals;%s \\[proof-layout-windows] refreshes"
+    (if (not proof-multiple-frames-enable) ;; and not proof-three-window-enable? 
+	(format " \\[proof-display-some-buffers] rotates output%s;"
+		(if nextbuf (concat " (next:" nextbuf ")") ""))
+      ""))))
+
+(defun pg-jump-to-end-hint ()
+  (pg-hint "Use \\[proof-goto-end-of-locked] to jump to end of processed region"))
+  
+(defun pg-processing-complete-hint ()
+  "Display hint for showing end of locked region or processing complete."
+  (if (buffer-live-p proof-script-buffer)
+      (let ((win (get-buffer-window proof-script-buffer)))
+	(unless ;; end of locked already displayed
+	    (and win (pos-visible-in-window-p (proof-unprocessed-begin)))
+	  (save-excursion 
+	    (set-buffer proof-script-buffer)
+	    (cond
+	     ((proof-locked-region-empty-p)) ;; nothing if empty
+	     ((proof-locked-region-full-p)
+	      (pg-hint 
+	       (concat "Processing complete in " (buffer-name proof-script-buffer))))
+	     (t ;; partly complete: hint about displaying the locked end
+	      (pg-jump-to-end-hint))))))))
+
+(defun pg-hint (hintmsg)
+  "Display a hint HINTMSG in the minibuffer, if `pg-show-hints' is non-nil.
+The function `substitute-command-keys' is called on the argument."
+  (if pg-show-hints (message (substitute-command-keys hintmsg))))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; Identifier under mouse query  (added in PG 3.5)
+;;
+
+;; FIXME: making the binding globally is perhaps a bit obnoxious, but
+;; this modifier combination is currently unused.
+(cond 
+ (proof-running-on-Emacs21
+  (global-set-key [C-M-mouse-2] 'pg-identifier-under-mouse-query))
+ (proof-running-on-XEmacs
+  (global-set-key '(control meta button2) 'pg-identifier-under-mouse-query)))
+
+(defun pg-identifier-under-mouse-query (event)
+  (interactive "e")
+  (if proof-shell-identifier-under-mouse-cmd
+      (let (identifier ctxt oldend) 
+	(save-selected-window
+	  (save-selected-frame
+	    (save-excursion
+	      (if (event-window event)
+		  (progn
+		    (select-window (event-window event))
+		    (setq oldend (if (region-exists-p)
+				     (region-end)))))
+	      (mouse-set-point event)
+	      (setq identifier
+		    ;; If there's an active region in this buffer, use that
+		    ;; instead of the identifier under point.  Since
+		    ;; region-end moves immediately to new point with 
+		    ;; zmacs-regions we use oldend instead of current.
+		    (if (region-exists-p)
+			(buffer-substring (region-beginning) 
+					  (or oldend (region-end)))
+		     (setq identifier (current-word))))
+	      (setq ctxt (proof-buffer-syntactic-context)))))
+	(unless (or (null identifier)
+		    (string-equal identifier ""))
+	  (proof-shell-invisible-command 
+	   (if (stringp proof-shell-identifier-under-mouse-cmd)
+	       ;; simple customization
+	       (format proof-shell-identifier-under-mouse-cmd identifier)
+	     ;; buffer-syntactic context dependent, as an alist
+	     ;; (handy for Isabelle: not a true replacement for parsing)
+	     (format (nth 1 (assq ctxt proof-shell-identifier-under-mouse-cmd))
+		     identifier)))))))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; Imenu and Speedbar (added in PG 3.5)
+;;
+
+(eval-after-load "speedbar"
+  '(and proof-assistant-symbol ;; *should* be set by now
+	(speedbar-add-supported-extension 
+	 (nth 2 (assoc proof-assistant-symbol proof-assistant-table)))))
+
+(defun proof-imenu-enable ()
+  "Add or remove index menu."
+  (if proof-imenu-enable
+      (imenu-add-to-menubar "Index")
+    (if proof-running-on-XEmacs
+	(easy-menu-remove (list "Index" :filter 'imenu-menu-filter))
+      (progn
+	(let ((oldkeymap (keymap-parent (current-local-map))))
+	  (if ;; sanity checks in case someone else set local keymap
+	      (and oldkeymap
+		   (lookup-key (current-local-map) [menu-bar index])
+		   (not 
+		    (lookup-key oldkeymap [menu-bar index])))
+	      (use-local-map oldkeymap)))
+	(remove-hook 'menu-bar-update-hook 'imenu-update-menubar)))))
 
 (provide 'pg-user)
 ;; pg-user.el ends here.
