@@ -1,10 +1,10 @@
 ;; proof-compat.el   Operating system and Emacs version compatibility
 ;;
 ;; Copyright (C) 2000-2002 LFCS Edinburgh. 
-;; Author:      David Aspinall <da@dcs.ed.ac.uk> and others
+;; Author:      David Aspinall <David.Aspinall@ed.ac.uk> and others
 ;; License:     GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-compat.el,v 7.0 2002/08/29 09:12:46 da Exp
+;; proof-compat.el,v 8.1 2004/04/18 10:34:40 da Exp
 ;;
 ;; This file collects together compatibility hacks for different
 ;; operating systems and Emacs versions.  This is to help keep
@@ -17,6 +17,40 @@
 
 (require 'proof-site)			; for architecture flags
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Emacs and XEmacs modifications and adjustments
+;;;
+
+;; Remove a custom setting.  Needed to support dynamic reconfiguration.
+;; (We'd prefer that repeated defcustom calls acted like repeated
+;;  "defvar treated as defconst" in XEmacs)
+(defun pg-custom-undeclare-variable (symbol)
+  "Remove a custom setting SYMBOL.
+Done by `makunbound' and removing all properties mentioned by custom library."
+  (mapcar (lambda (prop) (remprop symbol prop))
+	  '(default 
+	     standard-value 
+	     force-value 
+	     variable-comment
+	     saved-variable-comment
+	     variable-documentation
+	     group-documentation
+	     custom-set
+	     custom-get
+	     custom-options
+	     custom-requests
+	     custom-group
+	     custom-prefix
+	     custom-tag
+	     custom-links
+	     custom-version
+	     saved-value
+	     theme-value
+	     theme-face))
+  (makunbound symbol))
+
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; XEmacs compatibility
@@ -60,6 +94,19 @@ that use a window system such as X, and false for text-only terminals."
       (or (eq (console-type) 'x)
 	  (eq (console-type) 'mswindows))))
 
+(or (fboundp 'subst-char-in-string)
+;; Code is taken from Emacs 21.2.1/subr.el 
+(defun subst-char-in-string (fromchar tochar string &optional inplace)
+  "Replace FROMCHAR with TOCHAR in STRING each time it occurs.
+Unless optional argument INPLACE is non-nil, return a new string."
+  (let ((i (length string))
+	(newstr (if inplace string (copy-sequence string))))
+    (while (> i 0)
+      (setq i (1- i))
+      (if (eq (aref newstr i) fromchar)
+	  (aset newstr i tochar)))
+    newstr)))
+
 (or (fboundp 'replace-regexp-in-string)
 ;; Code is taken from Emacs 21.1.1/subr.el 
 (defun replace-regexp-in-string (regexp rep string &optional
@@ -120,12 +167,62 @@ and replace a sub-expression, e.g.
       (apply #'concat (nreverse matches))))))
 
 
+;; The GNU Emacs implementation of easy-menu-define has a very handy
+;; :visible keyword.  To use that when it's available, we set a
+;; constant to be :visible or :active
+
+(defconst menuvisiblep (if proof-running-on-Emacs21 :visible :active)
+  ":visible (on GNU Emacs) or :active (otherwise). 
+The GNU Emacs implementation of easy-menu-define has a very handy
+:visible keyword.  To use that when it's available, we use this constant.")
+
+ 
+(or (fboundp 'frame-parameter)
+    (defalias 'frame-parameter 'frame-property))
+
+(or (boundp 'window-size-fixed)
+    (defvar window-size-fixed nil 
+      "Fudged version of GNU Emacs' setting.  Completely ignored."))
+
+(or (fboundp 'window-text-height)
+    (defalias 'window-text-height 'window-text-area-height))
+
+(or (fboundp 'set-window-text-height)
+(defun set-window-text-height (window height)
+  "Sets the height in lines of the text display area of WINDOW to HEIGHT.
+This doesn't include the mode-line (or header-line if any) or any
+partial-height lines in the text display area.
+
+If WINDOW is nil, the selected window is used.
+
+Note that the current implementation of this function cannot always set
+the height exactly, but attempts to be conservative, by allocating more
+lines than are actually needed in the case where some error may be present."
+  (let ((delta (- height (window-text-height window))))
+    (unless (zerop delta)
+      (let ((window-min-height 1))
+	(if (and window (not (eq window (selected-window))))
+	    (save-selected-window
+	      (select-window window)
+	      (enlarge-window delta))
+	  (enlarge-window delta)))))))
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; GNU Emacs compatibility
+;;; GNU Emacs compatibility with XEmacs
 ;;;
 
+(unless (fboundp 'save-selected-frame)
+(defmacro save-selected-frame (&rest body)
+  "Execute forms in BODY, then restore the selected frame.
+The value returned is the value of the last form in BODY."
+  (let ((old-frame (gensym "ssf")))
+    `(let ((,old-frame (selected-frame)))
+       (unwind-protect
+           (progn ,@body)
+         (select-frame ,old-frame))))))
+
 ;; Chars (borrowed from x-symbol-emacs.el compatability file)
 
 (unless (fboundp 'characterp) (defalias 'characterp 'integerp))
@@ -289,6 +386,7 @@ The returned value is one of the following symbols:
 ;; Emulate a useful builtin from XEmacs.
 
 (or (fboundp 'remassq)
+;; NB: Emacs has assoc package with assq-delete-all function
 (defun remassq (key alist)
   "Delete any elements of ALIST whose car is `eq' to KEY.
 The modified ALIST is returned."
@@ -300,6 +398,68 @@ The modified ALIST is returned."
       (setq alist (cdr alist)))
     (nreverse newalist))))
 
+(or (fboundp 'remassoc)
+(defun remassoc (key alist)
+  "Delete any elements of ALIST whose car is `equal' to KEY.
+The modified ALIST is returned."
+;; The builtin version deletes by side-effect, but don't bother here.
+  (let (newalist)
+    (while alist
+      (unless (equal key (caar alist))
+	(setq newalist (cons (car alist) newalist)))
+      (setq alist (cdr alist)))
+    (nreverse newalist))))
+
+(or (fboundp 'frames-of-buffer)
+;; From XEmacs 21.4.12, aliases expanded
+(defun frames-of-buffer (&optional buffer visible-only)
+  "Return list of frames that BUFFER is currently being displayed on.
+If the buffer is being displayed on the currently selected frame, that frame
+is first in the list.  VISIBLE-ONLY will only list non-iconified frames."
+  (let ((list (get-buffer-window-list buffer))
+	(cur-frame (selected-frame))
+	next-frame frames save-frame)
+
+    (while list
+      (if (memq (setq next-frame (window-frame (car list)))
+		frames)
+	  nil
+	(if (eq cur-frame next-frame)
+	    (setq save-frame next-frame)
+	  (and
+	   (or (not visible-only)
+	       (frame-visible-p next-frame))
+	   (setq frames (append frames (list next-frame))))))
+	(setq list (cdr list)))
+
+    (if save-frame
+	(append (list save-frame) frames)
+      frames))))
+
+;; These functions are used in the intricate logic around
+;; shrink-to-fit.  
+
+;; window-leftmost-p, window-rightmost-p: my implementations
+(or (fboundp 'window-leftmost-p)
+    (defun window-leftmost-p (window)
+      (zerop (car (window-edges window)))))
+
+(or (fboundp 'window-rightmost-p)
+    (defun window-rightmost-p (window)
+      (>= (nth 2 (window-edges window))
+	  (frame-width (window-frame window)))))
+
+;; with-selected-windown from XEmacs 21.4.12
+(or (fboundp 'with-selected-window)
+(defmacro with-selected-window (window &rest body)
+  "Execute forms in BODY with WINDOW as the selected window.
+The value returned is the value of the last form in BODY."
+  `(save-selected-window
+     (select-window ,window)
+     ,@body)))
+
+
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -315,6 +475,61 @@ The modified ALIST is returned."
 	(funcall (get this-command 'completion-function)))))
       
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Attempt to harmonise pop-to-buffer behaviour 
+;;;
+
+(if proof-running-on-Emacs21
+    ;; NB: GNU Emacs version has fewer args
+    (defalias 'pg-pop-to-buffer 'pop-to-buffer))
+
+(if proof-running-on-XEmacs
+;; Version from XEmacs 21.4.12, with args to match GNU Emacs
+;; NB: GNU Emacs version has fewer args, we don't use ON-FRAME
+(defun pg-pop-to-buffer (bufname &optional not-this-window-p no-record on-frame)
+  "Select buffer BUFNAME in some window, preferably a different one.
+If BUFNAME is nil, then some other buffer is chosen.
+If `pop-up-windows' is non-nil, windows can be split to do this.
+If optional second arg NOT-THIS-WINDOW-P is non-nil, insist on finding
+another window even if BUFNAME is already visible in the selected window.
+If optional fourth arg is non-nil, it is the frame to pop to this
+buffer on.
+If optional third arg is non-nil, do not record this in switching history.
+(addition for PG).
+
+If `focus-follows-mouse' is non-nil, keyboard focus is left unchanged."
+  (let ((oldbuf (current-buffer))
+	buf window frame)
+    (if (null bufname)
+	(setq buf (other-buffer (current-buffer)))
+      (setq buf (get-buffer bufname))
+      (if (null buf)
+	  (progn
+	    (setq buf (get-buffer-create bufname))
+	    (set-buffer-major-mode buf))))
+    (push-window-configuration)
+    (set-buffer buf)
+    (setq window (display-buffer buf not-this-window-p on-frame))
+    (setq frame (window-frame window))
+    ;; if the display-buffer hook decided to show this buffer in another
+    ;; frame, then select that frame, (unless obeying focus-follows-mouse -sb).
+    (if (and (not focus-follows-mouse)
+	     (not (eq frame (selected-frame))))
+	(select-frame frame))
+    (unless no-record (record-buffer buf))
+    (if (and focus-follows-mouse
+	     on-frame
+	     (not (eq on-frame (selected-frame))))
+	(set-buffer oldbuf)
+      ;; select-window will modify the internal keyboard focus of XEmacs
+      (select-window window))
+    buf))
+);;; End XEmacs only
+
+  
+
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Old Emacs version compatibility
@@ -322,11 +537,6 @@ The modified ALIST is returned."
 
 ;; Create a menu from a customize group, for older/non-existent customize
 
-(or (fboundp 'customize-menu-create)
-(defun customize-menu-create (&rest args)
-  "Dummy function for PG; please upgrade your Emacs."
-  nil))
-
 (or (fboundp 'process-live-p)
 (defun process-live-p (obj)
   "Return t if OBJECT is a process that is alive"
@@ -334,6 +544,23 @@ The modified ALIST is returned."
        (memq (process-status obj) '(open run stop)))))
 
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; Compatibility with Custom library function to create a menu
+;;
+;; For some unfathomable reason, customize-menu-create goes
+;; wrong with PG groups on Emacs 21.  (It works with 'customize
+;; though).  We just disable it there. It's not worth this hassle.
+;;
+;; PG 3.5: this was used in proof-menu.el.  Things seem okay again 
+;; as of GNU Emacs 21.3.1.
+;; (cond
+;;  (proof-running-on-XEmacs
+;;   (defun pg-customize-menu-create (grp &optional name)
+;;     (list (customize-menu-create grp name))))
+;;  (t
+;;   (defun pg-customize-menu-create (grp &optional name)
+;;     nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
