@@ -1,12 +1,12 @@
 ;; proof-config.el  Proof General configuration for proof assistant
 ;;
-;; Copyright (C) 1998-2002 LFCS Edinburgh. 
-;; Author:      David Aspinall <da@dcs.ed.ac.uk> and others
+;; Copyright (C) 1998-2004 LFCS Edinburgh. 
+;; Author:      David Aspinall <David.Aspinall@ed.ac.uk> and others
 ;; License:     GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; Maintainer:  Proof General maintainer <proofgen@dcs.ed.ac.uk>
+;; Maintainer:  Proof General maintainer <da+pg-feedback@inf.ed.ac.uk>
 ;;
-;; proof-config.el,v 7.0 2002/08/29 09:12:46 da Exp
+;; proof-config.el,v 8.0 2004/04/17 23:39:59 da Exp
 ;;
 ;; This file declares all user options and prover-specific
 ;; configuration variables for Proof General.  The variables
@@ -99,8 +99,13 @@ terminator somewhere nearby.  Electric!"
   :group 'proof-user-options)
 
 (defcustom proof-toolbar-enable t
-  "*If non-nil, display Proof General toolbar for script buffers.
-NB: the toolbar is only available with XEmacs and GNU Emacs>=21."
+  "*If non-nil, display Proof General toolbar for script buffers."
+  :type 'boolean
+  :set 'proof-set-value
+  :group 'proof-user-options)
+
+(defcustom proof-imenu-enable nil
+  "*If non-nil, display Imenu menu of items for script buffers."
   :type 'boolean
   :set 'proof-set-value
   :group 'proof-user-options)
@@ -114,22 +119,37 @@ whether X-Symbol is installed in your Emacs."
   :set 'proof-set-value
   :group 'proof-user-options)
 
+(defpgcustom mmm-enable nil
+  "*Whether to use MMM Mode in Proof General for this assistant.
+MMM Mode allows multiple modes to be used in the same buffer.
+If you activate this variable, whether or not you really get MMM
+support depends on whether your proof assistant supports it."
+  :type 'boolean
+  :set 'proof-set-value
+  :group 'proof-user-options)
+
+(defcustom pg-show-hints t
+  "*Whether to display keyboard hints in the minibuffer."
+  :type 'boolean
+  :group 'proof-user-options)
+  
 (defcustom proof-output-fontify-enable t
   "*Whether to fontify output from the proof assistant.
 If non-nil, output from the proof assistant will be highlighted
 in the goals and response buffers.
-(This is providing font-lock-keywords have been set for the 
+\(This is providing font-lock-keywords have been set for the 
 buffer modes)."
   :type 'boolean
   :group 'proof-user-options)
 
-(defcustom proof-trace-output-fontify-enable t ;; testing
-  ;; (not (and proof-running-on-XEmacs (>= emacs-major-version 21))) production
-  "*Whether to fontify output from the proof assistant during tracing.
-If non-nil and proof-output-fontify-enable is also non-nil, 
-output from the proof assistant will be highlighted in the trace buffer.
-This is not recommended in XEmacs 21, since the font-lock parser 
-is easily overloaded by large tracing output."
+;; FIXME: next one could be integer value for catchup delay
+(defcustom proof-trace-output-slow-catchup t 
+  "*If non-nil, try to redisplay less often during frequent trace output.
+Proof General will try to configure itself to update the display
+of tracing output infrequently when the prover is producing rapid,
+perhaps voluminous, output.  This counteracts the situation that
+otherwise Emacs may consume more CPU than the proof assistant,
+trying to fontify and refresh the display as fast as output appears."
   :type 'boolean
   :group 'proof-user-options)
 
@@ -143,32 +163,20 @@ directly (instead, they may only be entered as part of the script).
 
 Clever or arrogant users may want to avoid this test, which is
 done if this `proof-strict-state-preserving' is turned off (nil)."
-  :type 'boolean
+  :type  'boolean
   :group 'proof-user-options)
 
-(defcustom proof-strict-read-only 
-  ;; For GNU Emacs, strict read only is buggy when used in
-  ;; conjunction with font-lock.
-  ;; The second conjunctive ensures that the expression is either
-  ;; `nil' or `strict' (and not 15!!).  
-  (and proof-running-on-XEmacs 'strict)
+(defcustom proof-strict-read-only 'strict
   "*Whether Proof General is strict about the read-only region in buffers.
 If non-nil, an error is given when an attempt is made to edit the
 read-only region.  If nil, Proof General is more relaxed (but may give
-you a reprimand!).
-
-If you change proof-strict-read-only during a session, you must 
-use the \"Restart\" button (or M-x proof-shell-restart) before
-you can see the effect in buffers.
-
-The default value for proof-strict-read-only depends on which
-version of Emacs you are using.  In GNU Emacs, strict read only is buggy
-when it used in conjunction with font-lock, so it is disabled by default."
-  :type 'boolean
+you a reprimand!)."
+  :type  'boolean
+  :set   'proof-set-value
   :group 'proof-user-options)
 
 
-(defcustom proof-three-window-mode nil
+(defcustom proof-three-window-enable nil
   "*Whether response and goals buffers have dedicated windows.
 If non-nil, Emacs windows displaying messages from the prover will not
 be switchable to display other windows.
@@ -182,15 +190,9 @@ Emacs automatically resizing windows between proof steps.
 
 If you use several frames (the same Emacs in several windows on the
 screen), you can force a frame to stick to showing the goals or
-response buffer.
-
-For single frame use this option may be inconvenient for
-experienced Emacs users."  
-  ;; Did say: 
-  ;; Moreover, this option may cause problems with multi-frame 
-  ;; use because of a bug.  
-  ;; but I can't find it as of 3.0pre201099.  
+response buffer."  
   :type 'boolean 
+  :set 'proof-set-value
   :group 'proof-user-options)
 
 (defcustom proof-multiple-frames-enable nil
@@ -202,8 +204,7 @@ the goals and response buffers, by altering the Emacs variable
   :set 'proof-set-value
   :group 'proof-user-options)
 
-(defcustom proof-delete-empty-windows 
-  nil
+(defcustom proof-delete-empty-windows nil
   "*If non-nil, automatically remove windows when they are cleaned.
 For example, at the end of a proof the goals buffer window will
 be cleared; if this flag is set it will automatically be removed.
@@ -214,6 +215,13 @@ selected frame will be automatically deleted."
   :type 'boolean
   :group 'proof-user-options)
 
+(defcustom proof-shrink-windows-tofit nil
+  "*If non-nil, automatically shrink output windows to fit contents.
+In single-frame mode, this option will reduce the size of the
+goals and response windows to fit their contents."
+  :type 'boolean
+  :group 'proof-user-options)
+
 (defcustom proof-toolbar-use-button-enablers 
   (not
    (or 
@@ -237,17 +245,6 @@ of unreliabilities with enablers there."
   :type 'boolean
   :group 'proof-user-options)
 
-; (defcustom proof-auto-retract 
-;   nil
-;   "*If non-nil, retract automatically when locked region is edited.
-; With this option active, the locked region will automatically be
-; unlocked when the user attempts to edit it.   To make use of this
-; option, proof-strict-read-only should be turned off.
-
-; Note: this feature has not been implemented yet, it is only an idea."
-;   :type 'boolean
-;   :group 'proof-user-options)
-
 (defcustom proof-query-file-save-when-activating-scripting 
   t
 "*If non-nil, query user to save files when activating scripting.
@@ -317,7 +314,9 @@ you should set `proof-tidy-response' to nil."
 
 (defcustom proof-experimental-features 
   ;; Turn on experimental features for pre-releases.
-  (if (string-match "pre" proof-general-version) t)
+  ;; (if (string-match "pre" proof-general-version) t)
+  t  ;; Version 3.5: features classed as experimental have
+     ;; actually been tested for a while, so we enable them.
   "*Whether to enable certain features regarded as experimental.
 Proof General includes a few features designated as \"experimental\".
 Enabling these will usually have no detrimental effects on using PG,
@@ -325,10 +324,7 @@ but the features themselves may be buggy.
 
 We encourage users to set this flag and test the features, but being
 aware that the features may be buggy (problem reports and 
-suggestions for improvements are welcomed).
-
-By default, experimental features are turned on in development
-releases and turned off in stable releases."
+suggestions for improvements are welcomed)."
   :type 'boolean
   :group 'proof-user-options)
 
@@ -336,20 +332,30 @@ releases and turned off in stable releases."
 
 (defcustom proof-follow-mode 'locked
   "*Choice of how point moves with script processing commands.
-One of the symbols: 'locked, 'follow, 'ignore.
+One of the symbols: 'locked, 'follow, 'followdown, 'ignore.
 
 If 'locked, point sticks to the end of the locked region.
 If 'follow, point moves just when needed to display the locked region end.
+If 'followdown, point if necessary to stay in writeable region
 If 'ignore, point is never moved after movement commands or on errors.
 
 If you choose 'ignore, you can find the end of the locked using
 `M-x proof-goto-end-of-locked'."
   :type '(choice
 	  (const :tag "Follow locked region" locked)
+	  (const :tag "Follow locked region down" followdown)
 	  (const :tag "Keep locked region displayed" follow)
 	  (const :tag "Never move" ignore))
   :group 'proof-user-options)
 
+;; TODO: the auto action might be improved a bit: for example,
+;; when scripting is turned off because another buffer is
+;; being retracted, we almost certainly want to retract
+;; the currently edited buffer as well (use case is somebody
+;; realising a change has to made in an ancestor file).
+;; In that case as well (supposing file being unlocked is
+;; an ancestore), it also seems unlikely that we need
+;; to query for saves.
 (defcustom proof-auto-action-when-deactivating-scripting nil
   "*If 'retract or 'process, do that when deactivating scripting.
 
@@ -417,7 +423,7 @@ signals to the remote host."
 ;;   e) all above with GNU Emacs and XEmacs.
 ;; But it's difficult to keep track of all that!  
 ;; Please report any bad/failing colour 
-;; combinations to proofgen@dcs.ed.ac.uk
+;; combinations to da+pg-feedback@inf.ed.ac.uk
 ;;
 ;; Some of these faces aren't used by default in Proof General,
 ;; but you can use them in font lock patterns for specific
@@ -507,7 +513,7 @@ both XEmacs 20.4 and Emacs 20.3's version of font-lock.")
 
 (defface proof-error-face 
   (proof-face-specs
-   (:background "salmon1" :bold t)
+   (:background "indianred1" :bold t)
    (:background "brown" :bold t)
    (:bold t))
   "*Face for error messages from proof assistant."
@@ -548,7 +554,7 @@ Warning messages can come from proof assistant or from Proof General itself."
 
 (defface proof-mouse-highlight-face
   (proof-face-specs
-   (:background "paleturquoise")
+   (:background "lightblue")
    (:background "darkslateblue")
    (:italic t))
   "*General mouse highlighting face."
@@ -709,28 +715,27 @@ If a function, it should return the command string to insert."
 
 (defconst proof-toolbar-entries-default
   `((state	"Display Proof State" "Display the current proof state" t 
-		proof-showproof-command)
-    (context	"Display Context" "Display the current context" t 
-		proof-context-command)
+				      proof-showproof-command)
+    (context	"Display Context"     "Display the current context" t 
+				      proof-context-command)
     (goal	"Start a New Proof"   "Start a new proof" t
-		proof-goal-command)
+				      proof-goal-command)
     (retract	"Retract Buffer"      "Retract (undo) whole buffer" t)
     (undo	"Undo Step"           "Undo the previous proof command" t)
     (delete	"Delete Step"         nil t)
     (next	"Next Step"           "Process the next proof command" t)
     (use	"Use Buffer"  	      "Process whole buffer" t)
     (goto	"Goto Point"	      "Process or undo to the cursor position" t)
-    (restart	"Restart Scripting"   "Restart scripting (clear all locked regions)" t)
     (qed	"Finish Proof"        "Close/save proved theorem" t
-		proof-save-command)
+				      proof-save-command)
     (lockedend  "Goto Locked End"     nil t)
-    (find	"Find Theorems"	      "Find theorems" t
-		proof-find-theorems-command)
+    (find	"Find Theorems"	      "Find theorems" t proof-find-theorems-command)
     (command    "Issue Command"	      "Issue a non-scripting command" t)
     (interrupt  "Interrupt Prover"    "Interrupt the proof assistant (warning: may break synchronization)" t)
+    (restart	"Restart Scripting"   "Restart scripting (clear all locked regions)" t)
     (visibility "Toggle Visibility"   nil t)
     (info	nil		      "Show online proof assistant information" t
-		proof-info-command)
+				      proof-info-command)
     (help	nil		      "Proof General manual" t))
 "Example value for proof-toolbar-entries.  Also used to define scripting menu.
 This gives a bare toolbar that works for any prover, providing the
@@ -835,6 +840,13 @@ or `proof-script-parse-function'."
   "Regular expression which matches end of commands in proof script.
 You should set this variable in script mode configuration.
 
+The end of the command is considered to be the end of the match
+of this regexp.  The regexp may include a nested group, which
+can be used to recognize the start of the following command
+(or white space).  If there is a nested group, the end of the
+command is considered to be the start of the nested group,
+i.e. (match-beginning 1), rather than (match-end 0).
+
 To configure command recognition properly, you must set at least one
 of these: `proof-script-sexp-commands', `proof-script-command-end-regexp', 
 `proof-script-command-start-regexp', `proof-terminal-char',
@@ -853,11 +865,13 @@ or `proof-script-parse-function'."
   :type 'string
   :group 'prover-config)
 
-(defcustom proof-script-use-old-parser t ;; nil ;;experiment and let folk complain 
+(defcustom proof-script-use-old-parser nil ;;experiment and let folk complain 
   "Whether to use the old parsing mechanism.
-This is a stop-gap option in Proof General 3.4 added for
-proof assistants which still depend on peculiarities of the old
-parsing functions. (Specific example: Isar relies on certain
+By default, this is set to nil in Proof General 3.5.  
+Please report any proof script parsing oddities to 
+da+pg@@inf.ed.ac.uk.
+
+(NB: Specific example where new parser fails: Isar relies on certain
 text being sent to prover which according to syntax configuration
 are comments; new parser does not send these currently.)"
   :type 'boolean
@@ -878,11 +892,12 @@ initial ``goal'' and the final ``save'' command."
   :type 'boolean
   :group 'prover-config)
 
-;; Unadvertised customization variable
-(defvar proof-script-fly-past-comments t
-  "*If non-nil, fly past comments when scripting, coalescing them into single spans.")
-
-
+(defcustom proof-script-fly-past-comments nil
+  "*If non-nil, fly past successive comments when scripting, coalescing into single spans.
+The default setting for this before PG 3.5 was t, now it is nil.  If you
+prefered the old behaviour, customize this variable to t."
+  :type 'boolean
+  :group 'proof-user-options)
 
 (defcustom proof-script-parse-function nil
   "A function which parses a portion of the proof script.
@@ -921,12 +936,15 @@ but you can set this variable to something else more precise if necessary."
   :type 'string
   :group 'proof-script)
 
-(defcustom proof-script-comment-end "\n"
+(defcustom proof-script-comment-end ""
   "String which ends a comment in the proof assistant command language.
-The script buffer's comment-end is set to a space plus this string.
+Should be an empty string if comments are terminated by end-of-line
+The script buffer's comment-end is set to a space plus this string,
+if it is non-empty.
+
 See also `proof-script-comment-start'.
 
-You should set this variable for reliable working of Proof General,"
+You should set this variable for reliable working of Proof General."
   :type 'string
   :group 'proof-script)
 
@@ -1002,11 +1020,11 @@ for `proof-script-next-entity-regexps' used for function menus."
 
 (defcustom proof-goal-with-hole-regexp nil 
   "Regexp which matches a command used to issue and name a goal.
-The name of the theorem is build from the variable
+The name of the theorem is built from the variable
 proof-goal-with-hole-result using the same convention as 
-query-replace-regexp. 
+for `query-replace-regexp'. 
 Used for setting names of goal..save regions and for default
-function-menu configuration in proof-script-find-next-entity.
+configuration of other modes (function menu, imenu).
 
 It's safe to leave this setting as nil."
   :type 'regexp
@@ -1120,6 +1138,15 @@ default."
   :type 'function
   :group 'proof-script)
 
+
+(defcustom proof-script-imenu-generic-expression nil
+  "Regular expressions to help find definitions and proofs in a script.
+Value for `imenu-generic-expression', see documentation of Imenu
+and that variable for details."
+  :type 'sexp
+  :group 'proof-script)
+
+
 ;; FIXME da: This next one is horrible.  We clearly would rather
 ;; have just proof-goal-command regexp instead.  This was born to solve
 ;; problem that Coq can have goals which look like definitions, etc.
@@ -1151,6 +1178,8 @@ proof-goal-command-regexp instead)."
 ;;  move the code of isar-global-save-command-p to proof-done-advancing.
 ;; FIXME da: sounds like a good idea, then that would give us a proper 
 ;;  handling of nested proofs?
+;; FIXME: Pierre:Careful: in coq V8 I now need a function to detect save 
+;;  command. Because Proof <term>. is a term, but not Proof with ...
 ;;
 (defcustom proof-really-save-command-p (lambda (span cmd) t)
   "Is this really a save command?
@@ -1346,6 +1375,14 @@ for scripting commands), unless activated-interactively is set."
   :type 'number
   :group 'proof-script)
 
+;; FIXME: perhaps remove this setting?  Users rather object to it.
+(defcustom proof-indent-pad-eol nil
+  "*Whether to add spaces to end of lines.
+If enabled, spaces are added to the end of lines in an attempt to improve
+the appearance of locked regions in XEmacs."
+  :type 'boolean
+  :group 'proof-script)
+
 (defcustom proof-indent-hang nil
   "Enable 'hanging' indentation for proof script."
   :type 'boolean
@@ -1392,21 +1429,6 @@ for scripting commands), unless activated-interactively is set."
   :group 'proof-script)
 
 
-(defcustom proof-font-lock-zap-commas nil
-  "If non-nil, enable a font-lock hack which unfontifies commas.
-If you fontify variables inside lists like [a,b,c] by matching
-on the brackets `[' and `]', you may take objection to the commas 
-being coloured as well.  In that case, enable this hack which
-will magically restore the commas to the default font for you.
-
-The hack is rather painful and forces immediate fontification of
-files on loading (no lazy, caching locking).  It is unreliable
-under GNU Emacs, to boot.
-
-LEGO and Coq enable it by tradition."
-  :type 'boolean
-  :group 'proof-script)
-
 (defcustom proof-script-font-lock-keywords nil
   "Value of font-lock-keywords used to fontify proof scripts.
 This is currently used only by proof-easy-config mechanism,
@@ -1415,6 +1437,20 @@ See also proof-{shell,resp,goals}-font-lock-keywords."
   :type 'sexp
   :group 'proof-script)
 
+(defcustom proof-script-syntax-table-entries nil
+  "List of syntax table entries for proof script mode.
+A flat list of the form
+ 
+  (CHAR SYNCODE CHAR SYNCODE ...)
+
+See doc of `modify-syntax-entry' for details of characters
+and syntax codes.
+
+At present this is used only by the `proof-easy-config' macro."
+  :type 'sexp
+  :group 'proof-script)
+
+
 
 ;;
 ;; Proof script context menu customization
@@ -1460,7 +1496,7 @@ See pg-user.el: pg-create-in-span-context-menu for more hints."
 Suggestion: this can be set in proof-pre-shell-start-hook from
 a variable which is in the proof assistant's customization
 group.  This allows different proof assistants to coexist
-(albeit in separate Emacs sessions)."
+\(albeit in separate Emacs sessions)."
   :type 'string
   :group 'proof-shell)
 
@@ -1490,9 +1526,9 @@ See also `proof-shell-init-cmd'."
 (defcustom proof-shell-init-cmd nil
    "The command for initially configuring the proof process.
 This command is sent to the process as soon as synchronization is gained
-(when an annotated prompt is first recognized).  It can be used to configure 
+\(when an annotated prompt is first recognized).  It can be used to configure 
 the proof assistant in some way, or print a welcome message
-(since output before the first prompt is discarded).
+\(since output before the first prompt is discarded).
 
 See also `proof-shell-pre-sync-init-cmd'."
    :type '(choice string (const nil))
@@ -1618,7 +1654,21 @@ setting to good effect.  If the proof assistant has more complex
 file dependencies then you should configure it to communicate with
 Proof General about the dependencies rather than using this setting."
   :type 'boolean
-  :group 'proof-shell)
+  :group 'proof-shell) ;; not really proof-shell
+
+(defcustom proof-cannot-reopen-processed-files nil
+  "Non-nil if the prover allows re-opening of already processed files.
+
+If the user has used Proof General to process a file incrementally,
+then PG will retain the spans recording undo history in the buffer
+corresponding to that file (provided it remains visited in Emacs).
+
+If the prover allows, it will be possible to undo to a position within
+this file.  If the prover does *not* allow this, this variable should
+be set non-nil, so that when a completed file is activated for 
+scripting (to do undo operations), the whole history is discarded."
+  :type 'boolean
+  :group 'proof-shell)  ;; not really proof shell
 
 ;; (defcustom  proof-shell-adjust-line-width-cmd nil
 
@@ -1633,8 +1683,9 @@ Proof General about the dependencies rather than using this setting."
 
 (defcustom proof-shell-prompt-pattern nil 
    "Proof shell's value for comint-prompt-pattern, which see.
-This pattern is just for interaction in comint (shell buffer).
-You don't really need to set it."
+NB!! This pattern is just for interaction in comint (shell buffer).
+You don't really need to set it.  The important one to set is
+`proof-shell-annotated-prompt-regexp', which see."
    :type 'regexp
    :group 'proof-shell)
 
@@ -1705,7 +1756,7 @@ See `proof-shell-error-regexp' and `proof-shell-interrupt-regexp'."
   "Regular expression which matches an error message, perhaps with line/column.
 Used by `proof-next-error' to jump to line numbers causing 
 errors during some batch processing of the proof assistant.
-(During \"manual\" script processing, script usually automatically
+\(During \"manual\" script processing, script usually automatically
 jumps to the end of the locked region)
 
 Match number 2 should be the line number, if present.
@@ -1720,13 +1771,13 @@ which is assumed to precede pg-next-error-regexp."
   "Used to locate a filename that an error message refers to.
 Used by `proof-next-error' to jump to locations causing 
 errors during some batch processing of the proof assistant.
-(During \"manual\" script processing, the script usually automatically
+\(During \"manual\" script processing, the script usually automatically
 jumps to the end of the locked region).
 
 Match number 2 should be the file name, if present.
 
 Errors must first be matched by `pg-next-error-regexp'
-(whether they contain a line number or not).  The response buffer
+\(whether they contain a line number or not).  The response buffer
 is then searched *backwards* for a regexp matching this variable,
 `pg-next-error-filename-regexp'.  (So if the
 filename appears after the line number, make the first regexp
@@ -1909,7 +1960,7 @@ to do syntax highlighting with font-lock."
   :group 'proof-shell)
 
 (defcustom proof-shell-set-elisp-variable-regexp nil
-  "Regexp matching output telling Proof General to set some variable.
+  "Matches output telling Proof General to set some variable.
 This allows the proof assistant to configure Proof General directly
 and dynamically.  
 
@@ -1948,6 +1999,14 @@ The matching string will be parsed as XML and then processed by
   :type '(choice nil regexp)
   :group 'proof-shell)
 
+;; FIXME: next one needs changing to be a function, or have function
+;; built from it.
+(defcustom proof-shell-issue-pgip-cmd nil
+  "Command sent to prover to process PGIP command in %s placeholder."
+  :type '(choice nil string)
+  :group 'proof-shell)
+
+
 ;; FIXME FIXME: this next one not yet used.  It's hard to interleave
 ;; commands with the ordinary queue anyway: the prover should
 ;; automatically output this information if it is enabled.
@@ -1959,7 +2018,7 @@ sent when a proof is completed."
   :group 'proof-shell)
 
 (defcustom proof-shell-theorem-dependency-list-regexp nil
-  "Regexp matching output telling Proof General about dependencies.
+  "Matches output telling Proof General about dependencies.
 This is to allow navigation and display of dependency information.
 The output from the prover should be a message with the form
 
@@ -1975,10 +2034,9 @@ This is an experimental feature, currently work-in-progress."
   :group 'proof-shell)
 
 (defcustom proof-shell-theorem-dependency-list-split nil
-  "Regexp used to split the list of strings which match
-`proof-shell-theorem-dependency-list-regexp'.  Used as
-an argument to `split-string'; nil defaults to whitespace.
-(This setting is necessary for provers which allow whitespace in
+  "Splits strings which match `proof-shell-theorem-dependency-list-regexp'.  
+Used as an argument to `split-string'; nil defaults to whitespace.
+\(This setting is necessary for provers which allow whitespace in
 the names of theorems/definitions/constants), see setting for
 Isabelle in isa/isa.el and isar/isar.el."
   :type '(choice nil regexp)
@@ -1991,8 +2049,31 @@ A string with %s replaced by the dependency name."
   :type 'string
   :group 'proof-shell)
 
+(defcustom proof-shell-identifier-under-mouse-cmd nil
+  "Command sent to the prover to query about an identifier under the mouse.
+This is typically a command used to print a theorem, constant, or whatever.
+A string with %s replaced by the identifier, or maybe an association
+list between values for `proof-buffer-syntactic-context' and strings."
+  :type '(or string (list (cons (choice 'nil 'string 'comment)  string)))
+  :group 'proof-shell)
+
+;; A feature that could be added to support Isabelle's tracing of
+;; tactics.  Seems to be little used, so probably not worth adding.
+;(defcustom proof-shell-interactive-input-regexp nil
+;  "Matches special interactive input request prompt from the prover.
+;A line which matches this regexp  but would otherwise be treated
+;as an ordinary response is instead treated as a prompt for
+;input to be displayed in the minibuffer.  The line which is
+;input is delivered directly to the proof assistant.
+
+;This can be used if the proof assistant requires return key
+;presses to continue, or similar."
+;  :type 'string
+;  :group 'proof-shell)
+
+
 (defcustom proof-shell-trace-output-regexp nil
-  "Regexp matching tracing output which should be displayed in trace buffer.
+  "Matches tracing output which should be displayed in trace buffer.
 Each line which matches this regexp but would otherwise be treated
 as an ordinary response, is sent to the trace buffer instead of the
 response buffer.   
@@ -2007,6 +2088,14 @@ Suggestion: this can be set a function called by `proof-pre-shell-start-hook'."
   :group 'proof-shell)
 
 
+(defcustom proof-shell-thms-output-regexp nil
+  "Matches theorem display which should be displayed in theorem buffer.
+Each line which matches this regexp but would otherwise be treated
+as an ordinary response, is sent to the theorem buffer as well as the
+response buffer."
+  :type '(choice nil regexp)
+  :group 'proof-shell)
+
 ;;
 ;; 5c. hooks and other miscellaneous customizations
 ;;
@@ -2050,9 +2139,13 @@ We do not force pipes everywhere because this risks loss of data."
 
 (defcustom proof-shell-strip-crs-from-input t
   "If non-nil, replace carriage returns in every input with spaces.
-This is enabled by default: it is appropriate for some systems
-because several CR's can result in several prompts, which may mess
-up the display (or even worse, the synchronization)."
+This is enabled by default: it is appropriate for many systems
+based on human input, because several CR's can result in several 
+prompts, which may mess up the display (or even worse, the 
+synchronization).  
+
+If the prover can be set to output only one prompt for every chunk of
+input, then newlines can be retained in the input."
   :type 'boolean
   :group 'proof-shell)
   
@@ -2084,6 +2177,8 @@ suggests what class of command is about to be inserted:
  'proof-done-invisible	     A non-scripting command
  'proof-done-advancing	     A \"forward\" scripting command
  'proof-done-retracting	     A \"backward\" scripting command
+ 'init-cmd		     Initialization command sent to prover
+ 'interactive-input	     Special interactive input direct to prover
 
 Caveats: You should be very careful about setting this hook.  Proof
 General relies on a careful synchronization with the process between
@@ -2125,10 +2220,17 @@ its friends configured in the function proof-shell-start."
   :group 'proof-shell)
 
 (defcustom proof-shell-handle-error-or-interrupt-hook
-  '(proof-goto-end-of-locked-if-pos-not-visible-in-window)
+  '(proof-goto-end-of-locked-on-error-if-pos-not-visible-in-window)
   "Run after an error or interrupt has been reported in the response buffer.
 Hook functions may inspect `proof-shell-error-or-interrupt-seen' to 
-determine whether the cause was an error or interrupt."
+determine whether the cause was an error or interrupt.  Possible
+values for this hook include:
+
+ proof-goto-end-of-locked-on-error-if-pos-not-visible-in-window
+ proof-goto-end-of-locked-if-pos-not-visible-in-window
+
+which move the cursor in the scripting buffer on an error or
+error/interrupt."
   :type '(repeat function)
   :group 'proof-shell)
 
@@ -2180,6 +2282,18 @@ See also proof-{script,resp,goals}-font-lock-keywords."
   :type 'sexp
   :group 'proof-shell)
 
+(defcustom proof-shell-syntax-table-entries nil
+  "List of syntax table entries for proof script mode.
+A flat list of the form
+ 
+  (CHAR SYNCODE CHAR SYNCODE ...)
+
+See doc of `modify-syntax-entry' for details of characters
+and syntax codes.
+
+At present this is used only by the `proof-easy-config' macro."
+  :type 'sexp
+  :group 'proof-shell)
 
 
 ;;
@@ -2420,7 +2534,7 @@ of `easy-menu-define' for more details."
 Completion is activated with \\[complete].
 
 If this table is empty or needs adjusting, please make changes using
-`customize-variable' and send suggestions to support@@proofgeneral.org"
+`customize-variable' and send suggestions to da+pg-support@@inf.ed.ac.uk"
   :type '(list string)
   :group 'prover-config)
 
@@ -2443,7 +2557,7 @@ If this table is empty or needs adjusting, please make changes using
   :group 'proof-general-internals)
 
 (defcustom proof-general-home-page
-  "http://www.proofgeneral.org"
+  "http://proofgeneral.inf.ed.ac.uk"
   "*Web address for Proof General"
   :type 'string
   :group 'proof-general-internals)
