@@ -4,9 +4,11 @@
 ;; Authors:   David Aspinall
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-menu.el,v 7.0 2002/08/29 09:12:49 da Exp
+;; proof-menu.el,v 8.0 2004/04/17 23:39:59 da Exp
 ;;
 
+(require 'proof)
+(require 'proof-syntax)
 (require 'proof-toolbar)     ; needed for proof-toolbar-scripting-menu
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -17,15 +19,20 @@
 (defvar proof-display-some-buffers-count 0)
 
 (defun proof-display-some-buffers ()
-  "Display the reponse, goals, trace, or shell buffer, rotating.
+  "Display the reponse, trace, goals, or shell buffer, rotating.
 A fixed number of repetitions of this command switches back to
 the same buffer.
 Also move point to the end of the response buffer if it's selected.
-If in three window or multiple frame mode, display two buffers."
+If in three window or multiple frame mode, display two buffers.
+The idea of this function is to change the window->buffer mapping 
+without adjusting window layout."
   (interactive)
   ;; The GUI-tessence here is to implement a humane toggle, which
   ;; allows habituation.  E.g. two taps of C-c C-l always
   ;; shows the goals buffer, three the trace buffer, etc.
+  ;; (That behaviour makes less sense from the menu, though,
+  ;;  where it seems more natural just to rotate from last 
+  ;;  position)
   (cond 
    ((and (interactive-p) 
 	 (eq last-command 'proof-display-some-buffers))
@@ -33,25 +40,35 @@ If in three window or multiple frame mode, display two buffers."
    (t
     (setq proof-display-some-buffers-count 0)))
   (let* ((assocbufs   (remove-if-not 'buffer-live-p 
-				    (list proof-response-buffer
-					  proof-goals-buffer
-					  proof-trace-buffer
-					  proof-shell-buffer)))
-	(selectedbuf (nth (mod proof-display-some-buffers-count 
-			       (length assocbufs)) assocbufs)))
-    (cond
-     ((or proof-three-window-mode proof-multiple-frames-enable)
-      ;; Display two buffers: next in rotation and goals/response
-      ;; FIXME: this doesn't work as well as it might.
-      (proof-switch-to-buffer selectedbuf 'noselect)
-      (proof-switch-to-buffer (if (eq selectedbuf proof-goals-buffer)
-				  proof-response-buffer
-				proof-goals-buffer) 'noselect))
-     (selectedbuf
-      (proof-switch-to-buffer selectedbuf 'noselect)))
-    (if (eq selectedbuf proof-response-buffer)
-	(set-window-point (get-buffer-window proof-response-buffer)
-			  (point-max)))))
+				     (list proof-response-buffer
+					   proof-thms-buffer
+					   proof-trace-buffer
+					   proof-goals-buffer
+					   )))
+					;proof-shell-buffer
+	 (numassoc    (length assocbufs)))
+    ;; If there's no live other buffers, we don't do anything.
+    (unless (zerop numassoc)
+      (let
+	 ((selectedbuf (nth (mod proof-display-some-buffers-count 
+				 numassoc) assocbufs))
+	  (nextbuf     (nth (mod (1+ proof-display-some-buffers-count)
+				 numassoc) assocbufs)))
+	(cond
+	 ((or proof-three-window-enable proof-multiple-frames-enable)
+	  ;; Display two buffers: next in rotation and goals/response
+	  ;; FIXME: this doesn't work as well as it might.
+	  (proof-switch-to-buffer selectedbuf 'noselect)
+	  (proof-switch-to-buffer (if (eq selectedbuf proof-goals-buffer)
+				      proof-response-buffer
+				    proof-goals-buffer) 'noselect))
+	 (selectedbuf
+	  (proof-switch-to-buffer selectedbuf 'noselect)))
+	(if (eq selectedbuf proof-response-buffer)
+	    (set-window-point (get-buffer-window proof-response-buffer t)
+			      (point-max)))
+	(pg-hint (pg-response-buffers-hint (buffer-name nextbuf)))))))
+
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -74,7 +91,8 @@ If in three window or multiple frame mode, display two buffers."
 ;; C-c C-c is proof-interrupt-process in universal-keys
 (define-key map [(control c) (control f)] 'proof-find-theorems)
 (define-key map [(control c) (control h)] 'proof-help)
-(define-key map [(control c) (control l)] 'proof-display-some-buffers)
+(define-key map [(control c) (control o)] 'proof-display-some-buffers)
+(define-key map [(control c) (control l)] 'proof-layout-windows)
 (define-key map [(control c) (control n)] 'proof-assert-next-command-interactive)
 (define-key map [(control c) (control p)] 'proof-prf)
 (define-key map [(control c) (control r)] 'proof-retract-buffer)
@@ -132,44 +150,52 @@ If in three window or multiple frame mode, display two buffers."
 ;;;###autoload
 (defun proof-menu-define-specific ()
   (easy-menu-define 
-   proof-assistant-menu 
+    proof-assistant-menu 
     proof-mode-map
     (concat "The menu for " proof-assistant)
     (cons proof-assistant
-	  (append
-	   (proof-ass menu-entries)
-	   '("----")
-	   (or proof-menu-favourites
-	       (proof-menu-define-favourites-menu))
-	   (or proof-menu-settings
-	       (proof-menu-define-settings-menu))
-	   '("----")
-	   (list
-	    (vector
-	     (concat "Start " proof-assistant)
-	     'proof-shell-start
-	     ':active '(not (proof-shell-live-buffer)))
+	    (append
+	     (proof-ass menu-entries)
+	     '("----")
+	     (or proof-menu-favourites
+		 (proof-menu-define-favourites-menu))
+	     (or proof-menu-settings
+		 (proof-menu-define-settings-menu))
+	     '("----")
+	    (list
+	     (vector
+	      (concat "Start " proof-assistant)
+	      'proof-shell-start
+	      ':active '(not (proof-shell-live-buffer)))
 	    (vector
 	     (concat "Exit " proof-assistant)
 	     'proof-shell-exit
 	     ':active '(proof-shell-live-buffer)))
-	   '("----")
-	   (list
-	    (cons "Help"
-		  (append 
-		   `(;; FIXME: should only put these two on the
-		     ;; menu if the settings are non-nil.
-		     [,(concat proof-assistant " information")
-		      (proof-help) t]
-		     [,(concat proof-assistant " web page")
-		      (browse-url proof-assistant-home-page) t])
-		   (proof-ass help-menu-entries))))))))
-    
+	    '("----")
+	    (list
+	     (cons "Help"
+		   (append
+		    `([,(concat proof-assistant " information")
+		       '(proof-help)
+		       ,menuvisiblep proof-info-command]
+		      [,(concat proof-assistant " web page")
+		       '(browse-url proof-assistant-home-page)
+		       ,menuvisiblep proof-assistant-home-page])
+		    (proof-ass help-menu-entries))))))))
+
+(defun proof-assistant-menu-update ()
+  "Update proof assistant menu in scripting buffers."
+  (proof-map-buffers (proof-buffers-in-mode proof-mode-for-script)
+     (proof-menu-define-settings-menu)
+     (proof-menu-define-specific)
+     (easy-menu-remove proof-assistant-menu)
+     (easy-menu-add proof-assistant-menu proof-mode-map)))
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Contents of the menus
-;;;
+;;
+;; Contents of sub menus
+;;
 
 (defvar proof-help-menu
   '("Help"
@@ -180,100 +206,148 @@ If in three window or multiple frame mode, display two buffers."
   "Proof General help menu.")
 
 (defvar proof-show-hide-menu
-  '(("Show all"
+  '(("Show All"
      ["Proofs"    (pg-show-all-portions "proof") t]
      ["Comments"  (pg-show-all-portions "comment") t])
-    ("Hide all"
+    ("Hide All"
      ["Proofs"    (pg-show-all-portions "proof" 'hide) t]
      ["Comments"  (pg-show-all-portions "comment" 'hide) t]))
   "Show/hide submenu.")
 
 (defvar proof-buffer-menu
   (cons "Buffers"
-	'(["Active Scripting"
-	    (proof-switch-to-buffer proof-script-buffer)
-	    :active (buffer-live-p proof-script-buffer)]
-	   ["Rotate Output Buffers"
-	    proof-display-some-buffers
-	    :active (buffer-live-p proof-goals-buffer)]
-	   ;;["Goals"
-	   ;; (proof-switch-to-buffer proof-goals-buffer t)
-	   ;; :active (buffer-live-p proof-goals-buffer)]
-	   ;;["Response"
-	   ;; (proof-switch-to-buffer proof-response-buffer t)
-	   ;; :active (buffer-live-p proof-response-buffer)]
-	   ;;["Shell"
-	   ;; (proof-switch-to-buffer proof-shell-buffer)
-	   ;; :active (buffer-live-p proof-shell-buffer)])
-	   ;; FIXME: this next test doesn't work since menus
-	   ;; loaded before proof-shell-trace-output-regexp is
-	   ;; set (in proof-shell hook).  Should be better with
-	   ;; simplified customization mechanism.
-	   ;; ( if proof-shell-trace-output-regexp ... )
-	   ;;'(["Trace"
-	   ;;(proof-switch-to-buffer proof-trace-buffer)
-	   ;;:active (buffer-live-p proof-trace-buffer)])
-	   ["Clear Responses"
-	    pg-response-clear-displays
-	    :active (buffer-live-p proof-response-buffer)]))
+	'(["Layout Windows"
+	   proof-layout-windows]
+	  ["Rotate Output Buffers"
+	   proof-display-some-buffers
+	   :active (buffer-live-p proof-goals-buffer)]
+	  ["Clear Responses"
+	   pg-response-clear-displays
+	   :active (buffer-live-p proof-response-buffer)]
+	  "----"
+	  ["Active Scripting"
+	   (proof-switch-to-buffer proof-script-buffer)
+	   :active (buffer-live-p proof-script-buffer)]
+	  ["Goals"
+	   (proof-switch-to-buffer proof-goals-buffer t)
+	   :active (buffer-live-p proof-goals-buffer)]
+	  ["Response"
+	   (proof-switch-to-buffer proof-response-buffer t)
+	   :active (buffer-live-p proof-response-buffer)]
+	  ;; FIXME: next test doesn't work: menus are loaded before
+	  ;; proof-shell-trace-output-regexp is set (in proof-shell hook).  
+	  ;; Should be better with simplified customization mechanism.
+	  ;; ( if proof-shell-trace-output-regexp ... )
+	  ["Trace"
+	   (proof-switch-to-buffer proof-trace-buffer)
+	   :active (buffer-live-p proof-trace-buffer)]
+	  ["Shell"
+	   (proof-switch-to-buffer proof-shell-buffer)
+	   :active (buffer-live-p proof-shell-buffer)]))
   "Proof General buffer menu.")
+	  
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; "Quick" (or main) options
+;;
 
-;; Make the togglers used in options menu below
+;; First, make the togglers used in options menu below
 
-(proof-deftoggle proof-three-window-mode)
 (proof-deftoggle proof-script-fly-past-comments)
 (proof-deftoggle proof-delete-empty-windows)
+(proof-deftoggle proof-shrink-windows-tofit)
 (proof-deftoggle proof-multiple-frames-enable proof-multiple-frames-toggle)
-(proof-deftoggle proof-output-fontify-enable proof-output-fontify-toggle)
+(proof-deftoggle proof-three-window-enable proof-three-window-toggle)
+;; (proof-deftoggle proof-output-fontify-enable proof-output-fontify-toggle)
 (proof-deftoggle proof-disappearing-proofs)
-(proof-deftoggle-fn (proof-ass-sym x-symbol-enable) 'proof-x-symbol-toggle)
+(proof-deftoggle proof-strict-read-only)
 
-(defvar proof-quick-opts-menu
+(proof-deftoggle-fn 'proof-imenu-enable 'proof-imenu-toggle)
+(proof-deftoggle-fn (proof-ass-sym x-symbol-enable) 'proof-x-symbol-toggle)
+(proof-deftoggle-fn (proof-ass-sym mmm-enable) 'proof-mmm-toggle)
+
+;; Here is the menu
+
+(defconst proof-quick-opts-menu
   (cons 
    "Options"
-   '(["Disppearing Proofs" proof-disappearing-proofs-toggle 
-      :style toggle
-      :selected proof-disappearing-proofs]
-     ["Three Window Mode" proof-three-window-mode-toggle
-      :active (not proof-multiple-frames-enable)
-      :style toggle
-      :selected proof-three-window-mode]
-     ["Delete Empty Windows" proof-delete-empty-windows-toggle
-      :active (not proof-multiple-frames-enable)
-      :style toggle
-      :selected proof-delete-empty-windows]
-     ["Multiple Frames" proof-multiple-frames-toggle
-      :active (display-graphic-p)
-      :style toggle
-      :selected proof-multiple-frames-enable]
-     ["Output Highlighting" proof-output-fontify-toggle
-      :active t
-      :style toggle
-      :selected proof-output-fontify-enable]
-     ["Toolbar" proof-toolbar-toggle
-      :active (and (or (featurep 'toolbar) (featurep 'tool-bar))
-		   (boundp 'proof-buffer-type)
-		   ;; only allow toggling of toolbar enable in one
-		   ;; buffer to avoid strange effects because we
-		   ;; only keep one flag.  (Strange effects because 
-		   ;; we only turn it off in one buffer at a time)
-		   (eq proof-buffer-type 'script))
-      :style toggle
-      :selected proof-toolbar-enable]
-     ["X-Symbol" proof-x-symbol-toggle
-      :active (proof-x-symbol-support-maybe-available)
-      :style toggle
-      :selected (proof-ass x-symbol-enable)]
+   `(["Electric Terminator" proof-electric-terminator-toggle
+     :style toggle
+     :selected proof-electric-terminator-enable]
      ["Fly Past Comments" proof-script-fly-past-comments-toggle
-      :active (not proof-script-use-old-parser)
+      ,menuvisiblep (not proof-script-use-old-parser)
       :style toggle
       :selected proof-script-fly-past-comments]
+     ["Disppearing Proofs" proof-disappearing-proofs-toggle 
+      :style toggle
+      :selected proof-disappearing-proofs]
+
+;     ["Output Highlighting" proof-output-fontify-toggle
+;      :active t
+;      :style toggle
+;      :selected proof-output-fontify-enable]
+
+     ["Strict Read Only" proof-strict-read-only-toggle
+      :style toggle
+      :selected proof-strict-read-only]
+
+     ;; X-Symbol and MM are minor modes which PG settings
+     ;; enable by default for PG buffers
+     ["X-Symbol"  (proof-x-symbol-toggle (if x-symbol-mode 0 1))
+      :active (proof-x-symbol-support-maybe-available)
+      :style toggle
+      :selected (and (boundp 'x-symbol-mode) x-symbol-mode)]
+     ["Multiple Modes" (proof-mmm-toggle (if mmm-mode 0 1))
+      :active (proof-mmm-support-available)
+      :style toggle
+      :selected (and (boundp 'mmm-mode) mmm-mode)]
+ 
+     ["Toolbar" proof-toolbar-toggle
+      ;; should really be split into :active & GNU Emacs's :visible
+      :active (and (or (featurep 'toolbar) (featurep 'tool-bar))
+			 (boundp 'proof-buffer-type)
+			 ;; only allow toggling of toolbar enable in one
+			 ;; buffer to avoid strange effects because we
+			 ;; only keep one flag.  (Strange effects because 
+			 ;; we only turn it off in one buffer at a time)
+			 (eq proof-buffer-type 'script))
+      :style toggle
+      :selected proof-toolbar-enable]
+     ["Index Menu" proof-imenu-toggle
+      :active (stringp (locate-library "imenu"))
+      :style toggle
+      :selected proof-imenu-enable]
+     ("Display"
+      ["Layout Windows" proof-layout-windows]
+      ["Use Three Panes" proof-three-window-toggle
+       :active (not proof-multiple-frames-enable)
+       :style toggle
+       :selected proof-three-window-enable]
+      ;; We use non-Emacs terminology "Windows" in this menu to help
+      ;; non-Emacs users.  Cf. Gnome usability studies: menus saying
+      ;; "Web Browser" more useful to novices than menus saying "Mozilla"!!
+      ["Multiple Windows" proof-multiple-frames-toggle
+       :active (display-graphic-p)
+       :style toggle
+       :selected proof-multiple-frames-enable]
+      ["Delete Empty Panes" proof-delete-empty-windows-toggle
+       :active (not proof-multiple-frames-enable)
+       :style toggle
+       :selected proof-delete-empty-windows]
+      ["Shrink to Fit" proof-shrink-windows-tofit-toggle
+       :active (not proof-multiple-frames-enable)
+       :style toggle
+       :selected proof-shrink-windows-tofit])
      ("Follow Mode" 
       ["Follow Locked Region" 
        (customize-set-variable 'proof-follow-mode 'locked)
        :style radio
        :selected (eq proof-follow-mode 'locked)]
+      ["Follow Locked Region Down" 
+       (customize-set-variable 'proof-follow-mode 'followdown)
+       :style radio
+       :selected (eq proof-follow-mode 'followdown)]
       ["Keep Locked Region Displayed" 
        (customize-set-variable 'proof-follow-mode 'follow)
        :style radio
@@ -282,24 +356,81 @@ If in three window or multiple frame mode, display two buffers."
        (customize-set-variable 'proof-follow-mode 'ignore)
        :style radio
        :selected (eq proof-follow-mode 'ignore)])
+     ;; Add this because it's a handy one to set (usually to retract)
+     ("Deactivate Action"
+      ["Retract" 
+       (customize-set-variable 'proof-auto-action-when-deactivating-scripting 'retract)
+       :style radio
+       :selected (eq proof-auto-action-when-deactivating-scripting 'retract)]
+      ["Process" 
+       (customize-set-variable 'proof-auto-action-when-deactivating-scripting 'process)
+       :style radio
+       :selected (eq proof-auto-action-when-deactivating-scripting 'process)]
+      ["Query"
+       (customize-set-variable 'proof-auto-action-when-deactivating-scripting nil)
+       :style radio
+       :selected (null proof-auto-action-when-deactivating-scripting)])
      "----"
-     ["Save Options" (proof-quick-opts-save) t]))
+     ["Reset Options" (proof-quick-opts-reset) 
+      (proof-quick-opts-changed-from-defaults-p)]
+     ["Save Options" (proof-quick-opts-save) 
+      (proof-quick-opts-changed-from-saved-p)]))
   "Proof General quick options.")
 
-(defun proof-quick-opts-save ()
-  "Save current values of PG Options menu items using Custom."
-  (interactive)
-  (pg-custom-save-vars
+(defun proof-quick-opts-vars ()
+  "Return a list of the quick option variables."
+  (list
+   'proof-electric-terminator-enable
+   'proof-script-fly-past-comments
    'proof-disappearing-proofs 
-   'proof-multiple-frames-enable
-   'proof-three-window-mode
+   ;;'proof-output-fontify-enable
+   'proof-strict-read-only
+   (proof-ass-sym x-symbol-enable)
+   (proof-ass-sym mmm-enable)
+   'proof-toolbar-enable
+   'proof-imenu-enable
+   ;; Display sub-menu
+   'proof-three-window-enable
    'proof-delete-empty-windows
    'proof-multiple-frames-enable
-   'proof-output-fontify-enable
-   'proof-toolbar-enable
-   'proof-script-fly-past-comments
-   (proof-ass-sym x-symbol-enable)
-   'proof-follow-mode))
+   'proof-shrink-windows-tofit
+   'proof-multiple-frames-enable
+   ;; Follow mode sub-menu
+   'proof-follow-mode
+   ;; Deactivate scripting action
+   proof-auto-action-when-deactivating-scripting))
+
+(defun proof-quick-opts-changed-from-defaults-p ()
+  ;; NB: would be nice to add.  Custom support?
+  t)
+
+(defun proof-quick-opts-changed-from-saved-p ()
+  ;; NB: would be nice to add.  Custom support?
+  t)
+
+
+;; 
+;; We have menu items for saving options and reseting them.
+;; We could just store the settings automatically (no save),
+;; but then the reset option would have to change to restore
+;; to manufacturer settings (rather then user-stored ones).
+;;
+(defun proof-quick-opts-save ()
+  "Save current values of PG Options menu items using custom."
+  (interactive)
+  (apply 'pg-custom-save-vars (proof-quick-opts-vars)))
+
+(defun proof-quick-opts-reset ()
+  "Reset PG Options menu to default (or user-set) values, using custom."
+  (interactive)
+  (apply 'pg-custom-reset-vars (proof-quick-opts-vars)))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; Main menu
+;;
+
 
 (defconst proof-config-menu
   (list "----"
@@ -311,27 +442,26 @@ If in three window or multiple frame mode, display two buffers."
 (defconst proof-advanced-menu
   (cons "Advanced..."
 	(append
-	 '(["Electric Terminator" proof-electric-terminator-toggle
-	    :style toggle
-	    :selected proof-electric-terminator-enable]
-	   ["Function Menu" function-menu
-	    :active (fboundp 'function-menu)]
-	   ["Complete Identifier" complete t]
-	   ["Next Error" proof-next-error
-	    :active pg-next-error-regexp])
+	 (if proof-running-on-XEmacs	;; speedbar not on standard menus
+	     '(["Speedbar" speedbar
+		:active (stringp (locate-library "speedbar"))]))
+	 '(["Complete Identifier" proof-script-complete t]
+	   ["Insert last output" pg-insert-last-output-as-comment proof-shell-last-output])
 	 (list "-----")
 	 proof-show-hide-menu
 	 (list "-----")
-	 ;; NB: customize-menu-create was buggy in earlier 
-	 ;; Emacs 21.X; okay since 21.1.1
+	 ;; NB: customize-menu-create is buggy in some versions of GNU Emacs
+	 ;; (bad in 21.1.0, good in 21.1.1, bad in 21.2.1, ...).  Comment
+	 ;; these next lines out if you must use one of these versions.
 	 (list (customize-menu-create 'proof-general))
-	 (list (customize-menu-create 'proof-general-internals 
-				      "Internals"))))
+	 (list (customize-menu-create 'proof-general-internals "Internals"))))
   "Advanced sub-menu of script functions and customize.")
 
 
 (defvar proof-menu  
-  '(["Scripting Active" proof-toggle-active-scripting
+  '(["Next Error" proof-next-error
+     :active pg-next-error-regexp]
+    ["Scripting Active" proof-toggle-active-scripting
      :style toggle
      :selected (eq proof-script-buffer (current-buffer))])
   "The Proof General generic menu for scripting buffers.")
@@ -392,26 +522,31 @@ If in three window or multiple frame mode, display two buffers."
 This is intended for defining proof assistant specific functions.
 STRING is inserted using `proof-insert', which see.
 KEY is added onto proof-assistant map."
-  (if key
-      (eval ;; eval-after-load "proof" ?
-       `(define-key (proof-ass keymap) (quote ,key) (quote ,fn))))
-  `(defun ,fn ()
-     ,(concat "Shortcut command to insert " string " into the current buffer.")
-     (interactive)
-     (proof-insert ,string)))
+  `(progn
+     (if ,key
+	 (define-key (proof-ass keymap) (quote ,key) (quote ,fn)))
+     (defun ,fn ()
+       ,(concat "Shortcut command to insert " 
+		(replace-in-string  string "\\\\" "\\\\=") ;; for substitute-command-keys
+		" into the current buffer.\nThis simply calls `proof-insert', which see.")
+       (interactive)
+       (proof-insert ,string))))
 
+;;;###autoload
 (defmacro proof-definvisible (fn string &optional key)
   "Define function FN to send STRING to proof assistant, optional keydef KEY.
 This is intended for defining proof assistant specific functions.
 STRING is sent using proof-shell-invisible-command, which see.
 KEY is added onto proof-assistant map."
-  (if key
-      (eval ;; eval-after-load "proof" ?
-       `(define-key (proof-ass keymap) (quote ,key) (quote ,fn))))
-  `(defun ,fn ()
-     ,(concat "Command to send " string " to the proof assistant.")
-     (interactive)
-     (proof-shell-invisible-command ,string)))
+  `(progn
+     (if ,key
+	 (define-key (proof-ass keymap) (quote ,key) (quote ,fn)))
+     (defun ,fn ()
+       ,(concat "Command to send " 
+		(replace-in-string  string "\\\\" "\\\\=") ;; for substitute-command-keys
+		" to the proof assistant.")
+       (interactive)
+       (proof-shell-invisible-command ,string))))
 
 (defun proof-def-favourite (command inscript menuname &optional key new)
   "Define and a \"favourite\" proof assisant function.
@@ -527,15 +662,22 @@ the form of the menu entry for the setting.")
   "Settings submenu for Proof General.")
 
 (defun proof-menu-define-settings-menu ()
-  "Return menu generated from `proof-assistant-settings'."
-  (let ((setgs proof-assistant-settings) ents)
-    (while setgs
-      (setq ents (cons 
-		  (apply 'proof-menu-entry-for-setting (car setgs)) ents))
-      (setq setgs (cdr setgs)))
-    (if ents
+  "Return menu generated from `proof-assistant-settings', update `proof-menu-settings'."
+  (if proof-assistant-settings
+      (let ((setgs proof-assistant-settings) 
+	    (save  (list "----"
+			 ["Reset Settings" (proof-settings-reset) 
+			  (proof-settings-changed-from-defaults-p)]
+			 ["Save Settings" (proof-settings-save)
+			  (proof-settings-changed-from-saved-p)]))
+	    ents)
+	(while setgs
+	  (setq ents (cons 
+		      (apply 'proof-menu-entry-for-setting (car setgs)) ents))
+	  (setq setgs (cdr setgs)))
 	(setq proof-menu-settings 
-	      (list (cons "Settings" ents))))))
+	      (list (cons "Settings" 
+			  (nconc ents save)))))))
 
 (defun proof-menu-entry-name (symbol)
   "Return a nice menu entry name for SYMBOL."
@@ -560,10 +702,34 @@ the form of the menu entry for the setting.")
 	      (proof-defstringset-fn pasym)
 	      t)))))
 
+(defun proof-settings-vars ()
+  "Return a list of proof assistant setting variables."
+  (mapcar (lambda (setting) (proof-ass-symv (car setting)))
+	  proof-assistant-settings))
+
+(defun proof-settings-changed-from-defaults-p ()
+  ;; FIXME: would be nice to add.  Custom support?
+  t)
+
+(defun proof-settings-changed-from-saved-p ()
+  ;; FIXME: would be nice to add.  Custom support?
+  t)
+
+(defun proof-settings-save ()
+  "Save current values of proof assistant settings using Custom."
+  (interactive)
+  (apply 'pg-custom-save-vars (proof-settings-vars)))
+
+(defun proof-settings-reset ()
+  "Reset proof assistant settings to their default values."
+  (interactive)
+  (apply 'pg-custom-reset-vars (proof-settings-vars)))
+
+
 ;;; autoload for compiled version: used in macro proof-defpacustom
 ;;;###autoload
 (defun proof-defpacustom-fn (name val args)
-  "As for macro `defpacustom' but evaluation arguments."
+  "As for macro `defpacustom' but evaluating arguments."
   (let (newargs setting evalform type)
     (while args
       (cond 
@@ -587,15 +753,23 @@ the form of the menu entry for the setting.")
 		      (eq (eval type) 'integer) 
 		      (eq (eval type) 'string)))
       (error "defpacustom: missing :type keyword or wrong :type value"))
+    ;; Debug message in case a defpacustom is repeated.
+    ;; NB: this *may* happen dynamically, but shouldn't: if the
+    ;; interface queries the prover for the settings it supports,
+    ;; then the internal list should be cleared first.
+    ;; FIXME: for now, we support redefinitions, by calling 
+    ;; pg-custom-undeclare-variable.
     (if (assoc name proof-assistant-settings)
-	(error "defpacustom: Proof assistanting setting %s already defined!"
-	       name))
+	(progn
+	  (proof-debug "defpacustom: Proof assistanting setting %s re-defined!"
+		       name)
+	  (undefpgcustom name)))
     ;; Could consider moving the bulk of the remainder of this
     ;; function to a function proof-assistant-setup-settings which
-    ;; defines the custom vals *and* menu entries.   This would
-    ;; allow proof assistant customization to manipulate
-    ;; proof-assistant-settings directly rather than forcing
-    ;; use of defpacustom.  (Probably stay as we are: more abstract)
+    ;; defines the custom vals *and* menu entries.  This would allow
+    ;; proof assistant customization to manipulate
+    ;; proof-assistant-settings directly rather than forcing use of
+    ;; defpacustom.  (Probably stay as we are: more abstract)
     (eval
      `(defpgcustom ,name ,val
 	,@newargs
@@ -610,7 +784,8 @@ the form of the menu entry for the setting.")
 	  (proof-assistant-invisible-command-ifposs
 	   (proof-assistant-settings-cmd (quote ,name))))))
     (setq proof-assistant-settings
-	  (cons (list name setting (eval type)) proof-assistant-settings))))
+	  (cons (list name setting (eval type)) 
+		(remassoc name proof-assistant-settings)))))
 
 ;;;###autoload
 (defmacro defpacustom (name val &rest args)
@@ -643,21 +818,29 @@ evaluate can be provided instead."
 	;;  Could also repeat last command if non-state destroying.
 	)))
 
+(defun proof-maybe-askprefs ()
+  "If `proof-assistant-settings' is unset, try to issue <askprefs>"
+  (if (and (not proof-assistant-settings)
+	   proof-shell-issue-pgip-cmd)
+      (pg-pgip-askprefs)))
+  
 
 (defun proof-assistant-settings-cmd (&optional setting)
-  "Return string for making setting vals kept in Proof General customizations.
+  "Return string for settings kept in Proof General customizations.
 If SETTING is non-nil, return a string for just that setting. 
-Otherwise return a string for configuring all settings."
-  (let
-      ((evalifneeded (lambda (expr)
-			(if (and (cadr expr) ;; setting has PA string?
-				 (or (not setting) 
-				     (eq setting (car expr))))
-			    (proof-assistant-format 
-			     (cadr expr) 
-			     (eval (proof-ass-symv (car expr))))))))
-    (apply 'concat (mapcar evalifneeded 
-			   proof-assistant-settings))))
+Otherwise return a string for configuring all settings.
+NB: if no settings are configured, this has no effect."
+  (if proof-assistant-settings
+      (let
+	  ((evalifneeded (lambda (expr)
+			   (if (and (cadr expr) ;; setting has PA string?
+				    (or (not setting) 
+					(eq setting (car expr))))
+			       (proof-assistant-format 
+				(cadr expr) 
+				(eval (proof-ass-symv (car expr))))))))
+	(apply 'concat (mapcar evalifneeded 
+			       proof-assistant-settings)))))
 
 (defun proof-assistant-format (string curvalue)
   "Replace a format characters %b %i %s in STRING by formatted CURVALUE.
@@ -693,7 +876,7 @@ NB: variable curvalue is dynamically scoped (used in proof-assistant-format).")
 (defun proof-assistant-format-string (value)
   (funcall proof-assistant-format-string-fn value))
 
-   
+
 
 
 (provide 'proof-menu)
