@@ -1,10 +1,10 @@
 ;; proof-toolbar.el    Toolbar for Proof General
 ;;
 ;; Copyright (C) 1998,9  David Aspinall / LFCS.
-;; Author:    David Aspinall <da@dcs.ed.ac.uk>
+;; Author:    David Aspinall <David.Aspinall@ed.ac.uk>
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-toolbar.el,v 7.0 2002/08/29 09:12:53 da Exp
+;; proof-toolbar.el,v 8.0 2004/04/17 23:39:59 da Exp
 ;;
 ;; NB: FSF GNU Emacs has no toolbar facility. This file defines
 ;; proof-toolbar-menu which holds the same commands and is put on the
@@ -137,7 +137,7 @@ For GNU Emacs, this holds a keymap.")
 ;;;###autoload
 (defun proof-toolbar-setup ()
   "Initialize Proof General toolbar and enable it for current buffer.
-If proof-mode-use-toolbar is nil, change the current buffer toolbar
+If `proof-toolbar-enable' is nil, change the current buffer toolbar
 to the default toolbar."
   (interactive)
   (if
@@ -217,17 +217,20 @@ to the default toolbar."
     (mapcar
      (lambda (buttons)
        (let ((var	(car buttons))
-	     (iconfiles (mapcar (lambda (name)
-				  (concat proof-images-directory
-					  name
-					  icontype)) (cdr buttons))))
+	     (iconfiles 
+	      (mapcar (lambda (name)
+			(concat proof-images-directory
+				"pg-" ;; added 1.3.04 to avoid Emacs clashes
+				name
+				icontype)) (cdr buttons))))
 	 (set var 
 	      (if proof-running-on-XEmacs
 		  ;; On XEmacs, icon variable holds a list of glyphs
 		  (toolbar-make-button-list iconfiles)
 		;; On GNU emacs, it holds a filename for the icon,
-		;; without path or extension.
-		(eval (cadr buttons))))))
+		;; without path or extension.  Warning!  This
+		;; can lead to name clashes with other packages.
+		(concat "pg-" (eval (cadr buttons)))))))
      proof-toolbar-icon-list)
 
     (if proof-running-on-XEmacs
