@@ -1,10 +1,10 @@
 ;; proof-utils.el   Proof General utility functions
 ;;
 ;; Copyright (C) 1998-2002 LFCS Edinburgh. 
-;; Author:      David Aspinall <da@dcs.ed.ac.uk> and others
+;; Author:      David Aspinall <David.Aspinall@ed.ac.uk> and others
 ;; License:     GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-utils.el,v 7.1 2002/08/29 10:06:43 da Exp
+;; proof-utils.el,v 8.0 2004/04/17 23:39:59 da Exp
 ;;
 ;;
 ;; Loading note: this file is required immediately from proof.el, so
@@ -158,6 +158,12 @@ If NEWVAL is present, set the variable, otherwise return its current value.")
 	    (setq ,specific-var newval)
 	  ,specific-var)))))
 
+(defun undefpgcustom (sym)
+  (let ((specific-var (proof-ass-symv sym))
+	(generic-var  (intern (concat "proof-assistant-" (symbol-name sym)))))
+    (pg-custom-undeclare-variable specific-var)
+    (fmakunbound generic-var)))
+
 (defmacro defpgcustom (sym &rest args)
   "Define a new customization variable <PA>-SYM for the current proof assistant.
 The function proof-assistant-<SYM> is also defined, which can be used in the 
@@ -249,13 +255,15 @@ Restrict to BUFLIST if it's set."
 ;;
 
 (defun pg-save-from-death ()
-  "Prevent this associated buffer from being killed.
+  "Prevent this associated buffer from being killed: merely erase it.
 A hook function for `kill-buffer-hook'.
 This is a fairly crude and not-entirely-robust way to prevent the 
 user accidently killing an associated buffer."
   (if (and (proof-shell-live-buffer) proof-buffer-type)
       (progn
 	(let ((bufname (buffer-name)))
+	  (erase-buffer)
+	  (set-buffer-modified-p nil)
 	  (bury-buffer)
 	  (error
 	   "Warning: buffer %s not killed; still associated with prover process."
@@ -297,7 +305,7 @@ The argument KBL is a list of tuples (k . f) where `k' is a keybinding
 (deflocal proof-font-lock-keywords nil
   "Value of font-lock-keywords in this buffer.
 We set `font-lock-defaults' to '(proof-font-lock-keywords t) for
-compatibility with X-Symbol, which may hack proof-font-lock-keywords
+compatibility with X-Symbol, which may hack `proof-font-lock-keywords'
 with extra patterns (in non-mule mode).")
 
 (deflocal proof-font-lock-keywords-case-fold-search nil
@@ -407,9 +415,7 @@ Returns new END value."
 	    (narrow-to-region start end)
 	    (run-hooks 'pg-before-fontify-output-hook)
 	    (setq end (point-max)))
-	  (font-lock-default-fontify-region start end nil)
-	  ;; FIXME: after-fontify hook might do this ugly zap commas thing.
-	  (proof-zap-commas-region start end))))
+	  (font-lock-default-fontify-region start end nil))))
   (save-restriction
     (narrow-to-region start end)
     (run-hooks 'pg-after-fontify-output-hook)
@@ -419,8 +425,9 @@ Returns new END value."
 	(pg-remove-specials start end)
 	(setq end (point))))
   (prog1
-      ;; Return new end value
-      (proof-x-symbol-decode-region start end)
+      ;; prog1 because we return new END value.
+      (if (proof-ass x-symbol-enable)
+	  (proof-x-symbol-decode-region start end))
     (proof-font-lock-clear-font-lock-vars)))
 
 
@@ -460,59 +467,81 @@ Default to whole buffer.  Leave point at END."
   (unless (and (boundp sym) (symbol-value sym))
     (warn "Proof General %s: %s is unset."  tag (symbol-name sym))))
 
+(defun proof-get-window-for-buffer (buffer)
+  "Find a window for BUFFER, display it there, return the window.
+NB: may change the selected window."
+  ;; IF there *isn't* a visible window showing buffer...
+  (unless (get-buffer-window buffer 'visible)
+    ;; THEN find somewhere nice to display it
+	  (if (and 
+	     ;; If we're in two-window mode and already displaying a
+	     ;; script/response/goals, try to just switch the buffer
+	     ;; instead of calling display-buffer which alters sizes.
+	     ;; Gives user some stability on display.
+	     (not proof-three-window-enable)
+	     (> (count-windows) 1)
+	     ;; was: (not (eq (next-window) (selected-window)))
+	     (memq (window-buffer (next-window nil 'ignoreminibuf))
+		   ;; NB: 3.5: added rest of assoc'd buffers here
+		   (cons proof-script-buffer (proof-associated-buffers))))
+	    (if (eq (selected-window) (minibuffer-window))
+		;; 17.8.01: avoid switching the minibuffer's contents 
+		;; -- terrrible confusion -- use next-window after
+		;; script buffer instead.
+		;; (another hack which is mostly right)
+		(set-window-buffer 
+		 (next-window 
+		  (car-safe (get-buffer-window-list proof-script-buffer))
+		  'ignoreminibuf) buffer)
+	      (if (eq (window-buffer (next-window nil 'ignoreminibuf))
+		      proof-script-buffer) 
+		  ;; switch this window
+		  (set-window-buffer (selected-window) buffer) 
+		;; switch other window
+		(set-window-buffer (next-window nil 'ignoreminibuf) buffer)))
+	  ;; o/w: call display buffer to configure windows nicely.
+	    (display-buffer buffer)))
+  ;; Return the window, hopefully the one we first thought of.
+  (get-buffer-window buffer 'visible))
+
 (defun proof-display-and-keep-buffer (buffer &optional pos)
-  "Display BUFFER and mark window according to `proof-three-window-mode'.
+  "Display BUFFER and make window according to display mode.
 If optional POS is present, will set point to POS.  
 Otherwise move point to the end of the buffer.
 Ensure that point is visible in window."
-  (let (window)
-    (save-excursion
-      (set-buffer buffer)
-      ;; Here's a hack: if we're asking to display BUFFER from a
-      ;; secondary window and the (next) other one is displaying the
-      ;; script buffer, then we do switch-buffer instead.  This means
-      ;; that goals and response buffer are swapped as expected in
-      ;; two-pane mode even if either one is used to "drive" the
-      ;; scripting.
-      ;; FIXME: would be better to deduce here which buffer
-      ;; we're displaying, and use get-buffer-window-list to do
-      ;; something sensible.
-      (if (and 
-	   (not proof-three-window-mode)
-	   (not (eq (next-window) (selected-window)))
-	   (eq (window-buffer (next-window nil 'ignoreminibuf))
-	       proof-script-buffer))
-	  (if (eq (selected-window) (minibuffer-window))
-	      ;; 17.8.01: avoid switching the minibuffer's contents 
-	      ;; -- terrrible confusion -- use next-window after
-	      ;; script buffer instead.
-	      ;; (another hack which is mostly right)
-	      (set-window-buffer 
-	       (next-window 
-		(car-safe (get-buffer-window-list proof-script-buffer))
-		'ignoreminibuf) buffer)
-	    (set-window-buffer (selected-window) buffer))
-	(display-buffer buffer))
-      (setq window (get-buffer-window buffer 'visible))
-      (set-window-dedicated-p window proof-three-window-mode)
-      (and window
-	   (save-selected-window
-	     (select-window window)
-	     ;; For various reasons, point may get moved
-	     ;; around in response buffer.
-	     (goto-char (or pos (point-max)))
-	     (if pos (beginning-of-line))
-	     ;; Ensure point visible
-	     (or (pos-visible-in-window-p (point) window)
-		 (recenter -1)))))))
+  (save-excursion
+    (save-selected-window
+      (let ((window (proof-get-window-for-buffer buffer)))
+	(if (window-live-p window) ;; [fails sometimes?]
+	    (progn
+	      ;; Set the size and point position.
+	      (set-window-dedicated-p window proof-three-window-enable)
+	      (select-window window)
+	      (if proof-shrink-windows-tofit
+		  (proof-resize-window-tofit)
+		;; If we're not shrinking to fit, allow the size of
+		;; this window to change.  [NB: might be nicer to
+		;; fix the size based on user choice]
+		(setq window-size-fixed nil))
+	      ;; For various reasons, point may get moved around in
+	      ;; response buffer.  Attempt to normalise its position.
+	      (goto-char (or pos (point-max)))
+	      (if pos 
+		  (beginning-of-line)
+		(skip-chars-backward "\n\t "))
+	      ;; Ensure point visible.  Again, window may have died
+	      ;; inside shrink to fit, for some reason
+	      (if (window-live-p window) 
+		  (unless (pos-visible-in-window-p (point) window)
+		    (recenter -1)))))))))
 
 (defun proof-clean-buffer (buffer)
   "Erase buffer and hide from display if proof-delete-empty-windows set.
 Auto deletion only affects selected frame.  (We assume that the selected
 frame is the one showing the script buffer.)"
   (with-current-buffer buffer
-    ;; NB: useful optional arg to erase buffer is XEmacs specific, 8-(.
     (erase-buffer)
+    (set-buffer-modified-p nil)
     (if (eq buffer proof-response-buffer)
 	(setq pg-response-next-error nil))	; all error msgs lost!
     (if proof-delete-empty-windows
@@ -530,19 +559,19 @@ The warning is coloured with proof-warning-face."
   (proof-display-and-keep-buffer proof-response-buffer))
 
 ;; could be a macro for efficiency in compiled code
-(defun proof-debug (&rest args)
-  "Issue the debugging messages ARGS in the response buffer, display it.
+(defun proof-debug (msg &rest args)
+  "Issue the debugging message (format MSG ARGS) in the response buffer, display it.
 If proof-show-debug-messages is nil, do nothing."
   (if proof-show-debug-messages
       (progn
-	(pg-response-display-with-face (apply 'concat 
-					      "PG debug: " 
-					      args)
+	(pg-response-display-with-face (concat "PG debug: " 
+					       (apply 'format msg args))
 				       'proof-debug-message-face)
 	(proof-display-and-keep-buffer proof-response-buffer))))
 
 
-;;; A handy utility function used in the "Buffers" menu.
+;;; A handy utility function used in the "Buffers" menu, and throughout
+;; the code.
 (defun proof-switch-to-buffer (buf &optional noselect)
   "Switch to or display buffer BUF in other window unless already displayed.
 If optional arg NOSELECT is true, don't switch, only display it.
@@ -552,8 +581,124 @@ No action if BUF is nil or killed."
   (and (buffer-live-p buf)
        (unless (eq buf (window-buffer (selected-window)))
 	 (if noselect
-	     (display-buffer buf t)
-	   (switch-to-buffer-other-window buf)))))
+	     ;; FIXME: would like 'norecord arg here to override
+	     ;; previous window entering top of MRU list here.
+	     ;; In fact, this could be hacked in XEmacs code.
+	     ;; GNU Emacs seems *not* to put previously displayed
+	     ;; window onto the top of the list with record-buffer:
+	     ;; that gives much nicer behaviour than XEmacs here.
+	     (display-buffer buf 'not-this-window)
+	   (let ((pop-up-windows t))
+	     (pg-pop-to-buffer buf 'not-this-window 'norecord))))))
+  
+
+;; Originally based on `shrink-window-if-larger-than-buffer', which
+;; has a pretty wierd implementation.
+;; TODO: desirable improvements would be to add some crafty history based
+;; on user resize-events.  E.g. user resizes window, that becomes the 
+;; new maximum size.
+;; FIXME: GNU Emacs has useful "window-size-fixed" which we use
+;; HOWEVER, it's still not quite the right thing, it seems to me.
+;; We'd like to specifiy a *minimum size* for a given buffer, 
+;; not a maximum.  With a frame split with just goals/response
+;; we'd still get resize errors here using window-size-fixed.
+;; FIXME: shrink-to-fit doesn't really work in three-buffer mode,
+;; since shrinking one of the associated buffers tends to enlarge the
+;; other (rather than just enlarging the proof state)
+(defun proof-resize-window-tofit (&optional window)
+  "Shrink the WINDOW to be as small as possible to display its contents.
+Do not shrink to less than `window-min-height' lines.
+Do nothing if the buffer contains more lines than the present window height,
+or if some of the window's contents are scrolled out of view,
+or if the window is not the full width of the frame,
+or if the window is the only window of its frame."
+;; da: actually seems okay in this case
+  (interactive)
+  (or window (setq window (selected-window)))
+  ;; some checks before resizing to avoid messing custom display
+  ;; [probably somewhat superfluous/extra rare]
+  (if 
+      (or
+       ;; The frame must not be minibuffer-only.
+       (eq (frame-parameter (window-frame window) 'minibuffer) 'only)
+       ;; We've got more than one window, right?
+       (= 1 (let ((frame (selected-frame)))
+	      (select-frame (window-frame window))
+	      (unwind-protect ;; [why is this protected?]
+		  (count-windows)
+		(select-frame frame)
+		(select-window window))))
+       ;; the window is the full width, right?
+       ;; [if not, we may be in horiz-split scheme, problematic]
+       (not (window-leftmost-p window))
+       (not (window-rightmost-p window)))
+      ;; OK, we're not going to adjust the height here.  Moreover,
+      ;; we'll make sure the height can be changed elsewhere.
+      (setq window-size-fixed nil)
+    (save-excursion
+      (set-buffer (window-buffer window))
+      (let* 
+	  ;; weird test cases:
+	  ;; cur=45, max=23, desired=121, extraline=0
+	  ;; current height
+	  ;;; ((cur-height (window-height window))
+	   ;; Most window is allowed to grow to
+	  ((max-height  
+	    (/ (frame-height (window-frame window)) 
+	       (if proof-three-window-enable 
+		   ;; we're in three-window-mode with
+		   ;; all horizontal splits, so share the height.
+		   3
+		 ;; Otherwise assume a half-and-half split
+		 2)))
+	   ;; If buffer ends with a newline, ignore it when counting
+	   ;; height unless point is after it.
+	   (extraline 
+	    (if (and (not (eobp))
+		     (eq ?\n (char-after (1- (point-max)))))
+		1 0))
+	   (test-pos (- (point-max) extraline))
+	   ;; Direction of resizing based on whether max position is 
+	   ;; currently visible.  [ FIXME: not completely sensible:
+	   ;; may be displaying end fraction of buffer! ]
+	   ;; (shrink (pos-visible-in-window-p test-pos window))
+	   ;; Likely desirable height is given by count-lines
+	   (desired-height 
+	    ;; FIXME: is count-lines too expensive for v.large
+	    ;; buffers?  Probably not an issue for us, but one
+	    ;; wonders at the shrink to fit strategy.
+	    ;; NB: way to calculate pixel fraction?
+	    (+ extraline 1 (count-lines (point-min) (point-max)))))
+	;; Let's shrink or expand.  Uses new GNU Emacs function.
+	(let ((window-size-fixed nil))
+	  (set-window-text-height window desired-height))
+;; 	(cond
+;; 	 ((and shrink
+;; 	       (> cur-height window-min-height)
+;; 	       ;; don't shrink if already too big; leave where it is
+;; 	       (< cur-height max-height))
+;; 	  (with-selected-window 
+;; 	   window
+;; 	   (shrink-window (- cur-height (max window-min-height desired-height)))))
+;; 	 (;; expand
+;; 	  (< cur-height max-height)
+;; 	  (with-selected-window window
+;; 	   (enlarge-window 
+;; 	    (- (min max-height desired-height) cur-height)))))
+	;; If we're at least the desirable height, it must be that the
+	;; whole buffer can be seen --- so make sure display starts at
+	;; beginning.
+	;; NB: shrinking windows can sometimes delete them
+	;; (although we don't want it to here!), but make this next 
+	;; check for robustness.
+	(if (window-live-p window)
+	    (progn
+	      (if (>= (window-height window) desired-height)
+		  (set-window-start window (point-min)))
+	      ;; window-size-fixed is a GNU Emacs buffer local variable 
+	      ;; which determines window size of buffer.
+	      (setq window-size-fixed (window-height window))))))))
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -568,14 +713,14 @@ No action if BUF is nil or killed."
       ((reporter-prompt-for-summary-p 
 	"(Very) brief summary of problem or suggestion: "))
     (reporter-submit-bug-report
-     "bugs@proofgeneral.org"
+     "da+pg-bugs@inf.ed.ac.uk"
      "Proof General" 
      (list 'proof-general-version 'proof-assistant)
      nil nil
      "[ When reporting a bug, please include a small test case for us to repeat it.
  Please also check that it is not already covered in the BUGS files that came with
  the distribution, or the latest versions at 
- http://www.proofgeneral.org/ProofGeneral/BUGS ]")))
+ http://proofgeneral.inf.ed.ac.uk/ProofGeneral/BUGS ]")))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -614,8 +759,9 @@ Args as for the macro `proof-defintset', except will be evaluated."
 	      ,(concat "Set `" (symbol-name var) "' to ARG.
 This function simply uses customize-set-variable to set the variable.
 It was constructed with `proof-defintset-fn'.")
-	      (interactive ,(concat "nValue for " (symbol-name var) 
-				    " (integer): "))
+	      (interactive ,(format "nValue for %s (int, currently %s):" 
+				     (symbol-name var)  
+				     (symbol-value var)))
 	      (customize-set-variable (quote ,var) arg))))
 
 (defmacro proof-defintset (var &optional othername)
@@ -634,8 +780,8 @@ Args as for the macro `proof-defstringset', except will be evaluated."
 	      ,(concat "Set `" (symbol-name var) "' to ARG.
 This function simply uses customize-set-variable to set the variable.
 It was constructed with `proof-defstringset-fn'.")
-	      (interactive ,(concat "sValue for " (symbol-name var) 
-				    " (a string): "))
+	      (interactive ,(format "sValue for %s (a string): "
+				    (symbol-name var)))
 	      (customize-set-variable (quote ,var) arg))))
 
 (defmacro proof-defstringset (var &optional othername)
@@ -666,7 +812,16 @@ The name of the defined function is returned."
 	(put symbol 'customized-value nil))))
   (custom-save-all))
 
-
+;; FIXME: this doesn't do quite same thing as reset button,
+;; which *removes* a setting from `custom-set-variables' list
+;; in custom.el.  Instead, this adds something to a 
+;; custom-reset-variables list.
+(defun pg-custom-reset-vars (&rest variables)
+  "Reset custom vars VARIABLES to their default values."
+  ;; FIXME: probably this XEmacs specific
+  (apply 'custom-reset-variables
+	 (mapcar (lambda (var) (list var 'default))
+		 variables)))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -688,7 +843,7 @@ EXTRAPATH is a list of extra path components"
        (executable-find progname)))	;; PG 3.4: try a new Emacs function.
     ((fboundp 'locate-file)
      (locate-file progname
-		  (append (split-path (getenv "PATH") extrapth)) 
+		  (append (split-path (getenv "PATH") extrapath)) 
 		  (if proof-running-on-win32 '(".exe"))
 		  1)))
    (if returnnopath progname)))
@@ -703,6 +858,8 @@ EXTRAPATH is a list of extra path components"
 (put 'proof-define-assistant-command 'lisp-indent-function 'defun)
 (put 'proof-define-assistant-command-witharg 'lisp-indent-function 'defun)
 (put 'defpgcustom 'lisp-indent-function 'defun)
+(put 'proof-map-buffers 'lisp-indent-function 'defun)
+(put 'proof-with-current-buffer-if-exists 'lisp-indent-function 'defun)
 
 (defconst proof-extra-fls
   (list
@@ -721,7 +878,7 @@ EXTRAPATH is a list of extra path components"
        nil t)))
 
 ;; This doesn't work for FSF's font lock, developers should use
-;; XEmacs!
+;; XEmacs!  (unless edebug is broken, which it is in my 21.4.12)
 (if (boundp 'lisp-font-lock-keywords)	; compatibility hack
     (setq lisp-font-lock-keywords 
 	  (append proof-extra-fls
