@@ -1,9 +1,9 @@
 ; isa-mode.el	 Emacs support for Isabelle proof assistant
 ;; Copyright (C) 1993-2000 LFCS Edinburgh, David Aspinall.
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
-;; Author:    David Aspinall <da@dcs.ed.ac.uk>
+;; Author:    David Aspinall <David.Aspinall@ed.ac.uk>
 ;;
-;; isa.el,v 7.1 2002/08/29 12:06:52 da Exp
+;; isa.el,v 8.0 2004/04/17 23:40:00 da Exp
 ;;
 ;; -----------------------------------------------------------------
 ;;
@@ -183,7 +183,7 @@ and script mode."
    ;; restoring settings saved by Proof General, etc.
 
    proof-shell-pre-sync-init-cmd	"ProofGeneral.init false;"
-   proof-shell-init-cmd		        (proof-assistant-settings-cmd)
+   proof-shell-init-cmd		        '(proof-assistant-settings-cmd)
 
    proof-shell-restart-cmd		"ProofGeneral.isa_restart();"
    proof-shell-quit-cmd			"quit();"
@@ -194,6 +194,11 @@ and script mode."
    proof-shell-eager-annotation-end     "\361\\|\363"
    ;; see isa-pre-shell-start for proof-shell-trace-output-regexp
 
+   ;; Isabelle is learning to talk PGIP...
+   proof-shell-match-pgip-cmd		"<pgip"
+   proof-shell-issue-pgip-cmd		
+	(if isa-supports-pgip 'isabelle-process-pgip nil)
+
    ;; Some messages delimited by eager annotations
    proof-shell-clear-response-regexp    "Proof General, please clear the response buffer."
    proof-shell-clear-goals-regexp       "Proof General, please clear the goals buffer."
@@ -224,6 +229,11 @@ and script mode."
    pg-subterm-help-cmd			"printyp (type_of (read \"%s\"))"
 
    ;; === MULTIPLE FILE HANDLING ===
+
+   ;; da: this next setting added for PG 3.5.  I think the theory
+   ;; loader changed at some point: originally this setting left as
+   ;; nil would have been okay.
+   proof-cannot-reopen-processed-files  t 
    proof-shell-process-file 
    (cons
     ;; Theory loader output and verbose update() output.
@@ -606,21 +616,6 @@ you will be asked to retract the file or process the remainder of it."
       (setq string (match-string 1 string))))
 
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;
-;; x-symbol support for Isabelle PG, provided by David von Oheimb.
-;;
-;; The following settings configure the generic PG package.
-;; The token language "Isabelle Symbols" is in file x-symbol-isabelle.el
-;;
-
-(setq proof-xsym-extra-modes '(thy-mode)
-      proof-xsym-activate-command
-      "print_mode := ([\"xsymbols\", \"symbols\"] @ ! print_mode);"
-      proof-xsym-deactivate-command
-      "print_mode := (! print_mode \\\\ [\"xsymbols\", \"symbols\"]);")
-
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; Completion table for Isabelle identifiers
