@@ -5,7 +5,7 @@
 ;; Author:      David Aspinall <da@dcs.ed.ac.uk>
 ;; Maintainer:  Proof General maintainer <proofgen@dcs.ed.ac.uk>
 ;;
-;; isabelle-system.el,v 7.1 2002/08/29 10:06:52 da Exp
+;; isabelle-system.el,v 8.0 2004/04/17 23:40:00 da Exp
 ;;
 ;; Most of this code is taken from the final version of Isamode.
 ;; --------------------------------------------------------------
@@ -40,12 +40,13 @@
       (proof-locate-executable "isatool")
       ;; FIXME: use same mechanism as isabelle-program-name below.
       (let ((possibilities
-	     '("/usr/bin/isatool"
-	       "/usr/share/Isabelle/bin/isatool"
-	       "/usr/local/bin/isatool"
-	       "/usr/local/Isabelle/bin/isatool"
-	       "/opt/bin/isatool"
-	       "/opt/Isabelle/bin/isatool")))
+	     (list
+	      (concat (getenv "HOME") "/Isabelle/bin/isatool")
+	      "/usr/share/Isabelle/bin/isatool"
+	      "/usr/local/bin/isatool"
+	      "/usr/local/Isabelle/bin/isatool"
+	      "/opt/bin/isatool"
+	      "/opt/Isabelle/bin/isatool")))
 	(while (and possibilities
 		    (not (file-executable-p
 			  (car possibilities))))
@@ -161,19 +162,35 @@ ISABELLE will always override this setting."
 		     (concat " " logic) "")))
     (concat isabelle opts logicarg)))
 
+
 (defun isabelle-choose-logic (logic)
   "Adjust isabelle-prog-name and proof-prog-name for running LOGIC."
-  ;; a little bit obnoxious maybe (but what naive user would expect)
+  (interactive 
+   (list (completing-read 
+	  "Use logic: "
+	  (mapcar 'list (cons "Default" 
+			      isabelle-logics-available)))))
+  ;; a little bit obnoxious maybe (but maybe what naive user would expect)
   ;; (customize-save-variable 'isabelle-chosen-logic logic)
-  (customize-set-variable 'isabelle-chosen-logic logic)
+  (if (proof-shell-live-buffer)
+      (error "Can't change logic while Isabelle is running, please exit process first!"))
+  (customize-set-variable 'isabelle-chosen-logic 
+			  (unless (string-equal logic "Default") logic))
   (setq isabelle-prog-name (isabelle-command-line))
-  (setq proof-prog-name isabelle-prog-name))
+  (setq proof-prog-name isabelle-prog-name)
+  ;; Settings are potentially different between logics, and
+  ;; so are Isar keywords.  Set these to nil so they get
+  ;; automatically re-initialised.  
+  ;; FIXME: Isar keywords change not handled yet.
+  (setq proof-assistant-settings nil)
+  (setq proof-menu-settings nil))
 
 (defun isa-tool-list-logics ()
   "Generate a list of available object logics."
   (if (isa-set-isatool-command)
-      (split-string (isa-shell-command-to-string
-		     (concat isa-isatool-command " findlogics")) "[ \t]")))
+      (delete "" (split-string 
+		  (isa-shell-command-to-string
+		   (concat isa-isatool-command " findlogics")) "[ \t]"))))
 
 (defun isa-view-doc (docname)
   "View Isabelle document DOCNAME, using Isabelle tools."
@@ -182,6 +199,29 @@ ISABELLE will always override this setting."
 	     "isa-view-doc" nil
 	     (list isa-isatool-command "doc" docname))))
 
+(defvar isabelle-version-string 'unset)
+
+(defun isa-version ()
+  "Try to retrieve a version value for Isabelle."
+  (unless (stringp isabelle-version-string)
+    (setq isabelle-version-string
+	  (if (isa-set-isatool-command)
+	      (isa-shell-command-to-string
+	       ;; This may return the string "Unknown Isabelle tool:
+	       ;; version", but that's fine.
+	       (concat isa-isatool-command " version"))
+	    "Unknown")))
+  isabelle-version-string)
+
+;; We're going to need to know this pretty soon, so let's do it now.
+(defconst isa-supports-pgip 
+  ;; PGIP-aware Isabelle versions are also aware of their own version
+  (not (string-match "^Unknown" (isa-version)))
+  "Whether the currently configured version of Isabelle supports PGIP.")
+
+      
+
+
 (defun isa-tool-list-docs ()
   "Generate a list of documentation files available, with descriptions.
 This function returns a list of lists of the form
@@ -192,13 +232,12 @@ passed to isa-tool-doc-command, DOCNAME will be viewed."
       (let ((docs (isa-shell-command-to-string
 		   (concat isa-isatool-command " doc"))))
 	(unless (string-equal docs "")
-	  (mapcar
+	  (mapcan
 	   (function (lambda (docdes)
-		       (if (proof-string-match "\\(\\S-+\\)[ \t]+" docdes)
-			   (list 
-			    (substring docdes (match-beginning 0) (match-end 1))
-			    (substring docdes (match-end 0)))
-			 '("???" "???"))))
+		       (if (proof-string-match "^[ \t]+\\(\\S-+\\)[ \t]+" docdes)
+			   (list (list 
+				  (substring docdes (match-beginning 0) (match-end 1))
+				  (substring docdes (match-end 0)))))))
 	   (split-string docs "\n"))))))
 
 (defun isa-quit (save)
@@ -227,6 +266,14 @@ Called with one argument: t to save database, nil otherwise."
 
 ;;; ==========  Utility functions ==========
 
+(defcustom isabelle-refresh-logics t
+  "*Whether to refresh the list of logics during an interactive session.
+If non-nil, then `isatool findlogics' will be used to regenerate
+the `isabelle-logics-available' setting.  If this tool does not work
+for you, you should disable this behaviour."
+  :type 'boolean
+  :group 'isabelle)
+
 (defcustom isabelle-logics-available (isa-tool-list-logics)
   "*List of logics available to use with Isabelle.
 If the `isatool' program is available, this is automatically
@@ -256,32 +303,70 @@ until Proof General is restarted."
   "Isabelle documentation menu.  Constructed when PG is loaded.")
 
 
+;; PG 3.5: logics menu is now refreshed automatically.  Rather
+;; a big effort for such a small effect.
 (defun isabelle-logics-menu-calculate ()
-  (cons "Logics" 
-	(cons
-	 ["Default" 
-	  (isabelle-choose-logic nil)
-	  :active (not (proof-shell-live-buffer))
-	  :style radio
-	  :selected (not isabelle-chosen-logic)]
-	 (mapcar (lambda (l) 
+  (setq isabelle-logics-menu-entries
+	(cons "Logics"
+	      (append
+	       '(["Default" 
+		  (isabelle-choose-logic nil)
+		  :active (not (proof-shell-live-buffer))
+		  :style radio
+		  :selected (not isabelle-chosen-logic)])
+	       (mapcar (lambda (l) 
 		   (vector l (list 'isabelle-choose-logic l)
 			   :active '(not (proof-shell-live-buffer))
 			   :style 'radio
 			   :selected (list 'equal 'isabelle-chosen-logic l)))
-		 (isa-tool-list-logics)))))
+		       isabelle-logics-available)))))
+
+(defvar isabelle-time-to-refresh-logics t
+  "Non-nil if we should refresh the logics list")
 
 (defun isabelle-logics-menu-refresh ()
-  "Refresh isabelle-logics-menu."
+  "Refresh isabelle-logics-menu-entries, returning new entries."
   (interactive)
-  (setq isabelle-logics-menu (isabelle-logics-menu-calculate))
-  ;; FIXME: need to do some easy-menu magic here
-  )
+  (if (and isabelle-refresh-logics
+	   (or isabelle-time-to-refresh-logics (interactive-p)))
+      (progn
+	(setq isabelle-logics-available (isa-tool-list-logics))
+	(isabelle-logics-menu-calculate)
+	(if proof-running-on-Emacs21 
+	    ;; update the menu manually
+	    (easy-menu-add-item proof-assistant-menu nil 
+				isabelle-logics-menu-entries))
+	(setq isabelle-time-to-refresh-logics nil) ;; just done it, don't repeat!
+	(run-with-timer 2 nil ;; short delay to avoid doing this too often
+			(lambda () (setq isabelle-time-to-refresh-logics t))))))
 
-(defconst isabelle-logics-menu (isabelle-logics-menu-calculate)
+;; Function for XEmacs only
+(defun isabelle-logics-menu-filter (&optional ignored)
+  (isabelle-logics-menu-refresh)
+  (cdr isabelle-logics-menu-entries))
+
+;; Functions for GNU Emacs only to update logics menu
+(if proof-running-on-Emacs21
+(defun isabelle-menu-bar-update-logics ()
+  ;; standard check we're being invoked
+  (and (current-local-map)
+       (keymapp (lookup-key (current-local-map) 
+ 			    (vector 'menu-bar (intern proof-assistant))))
+       (isabelle-logics-menu-refresh))))
+
+(if proof-running-on-Emacs21
+    (add-hook 'menu-bar-update-hook 'isabelle-menu-bar-update-logics))
+
+
+(defvar isabelle-logics-menu-entries (isabelle-logics-menu-calculate))
+
+(defvar isabelle-logics-menu 
+   (if proof-running-on-XEmacs
+       (cons (car isabelle-logics-menu-entries)
+	     (list :filter 'isabelle-logics-menu-filter)) ;; generates menu on click
+     isabelle-logics-menu-entries)
   "Isabelle logics menu.  Calculated when Proof General is loaded.")
 
-
 ;; Added in PG 3.4: load isar-keywords file.
 ;; This roughly follows the method given in the interface script.
 ;; It could be used to add an elisp command at the bottom of
@@ -309,6 +394,12 @@ until Proof General is restarted."
 ;; NB: use of defpacustom here gives  separate customizable
 ;; options for Isabelle and Isabelle/Isar.
 
+;; In latest release of Isabelle, these are set automatically via PGIP
+;; sent from Isabelle.
+
+;; BEGIN backwards compatibility 
+(cond 
+ ((not isa-supports-pgip)
 (defpacustom show-types  nil
   "Whether to show types in Isabelle."
   :type 'boolean
@@ -380,7 +471,8 @@ until Proof General is restarted."
 (defpacustom print-depth  10
   "Setting for the ML print depth in Isabelle."
   :type 'integer
-  :setting "print_depth %i;")
+  :setting "print_depth %i;")))
+;; END backwards compatibility 
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -462,5 +554,47 @@ the function `pg-remove-specials' can be used instead)."
 	       (not (string-equal thm proof-unnamed-theorem-name)))))))
 
 
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; XML as an SML string: add escapes for quotes
+;;
+
+(defun isabelle-xml-sml-escapes (xmlstring)
+  (replace-regexp-in-string "\"" "\\\"" xmlstring t t))
+
+(defun isabelle-process-pgip (xmlstring)
+  "Return an Isabelle or Isabelle/Isar command to process PGIP in XMLSTRING."
+  (if isa-supports-pgip
+      (let ((mlcmd (format "ProofGeneral.process_pgip(\"%s\");"
+			   (isabelle-xml-sml-escapes xmlstring))))
+	(if (eq proof-assistant-symbol 'isar)
+	    (isar-markup-ml mlcmd)
+	  mlcmd))
+    "")) ;; Empty string in case called with non PGIP-aware Isabelle.
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; Understanding syntax configuration (really should be PGIP, but...)
+;;
+;; [PRESENTLY UNUSED; WORK IN PROGRESS]
+
+(defun isabelle-parse-syntax-dump (buf)
+  (save-excursion
+    (let (consts start lim)
+      (set-buffer buf)
+      (goto-char (point-min))
+      (if (re-search-forward "consts:" nil t)
+	  (progn
+	    (setq start (point))
+	    (setq lim (re-search-forward "parse_ast_translation:" nil t))
+	    (goto-char start)
+	    (while (re-search-forward "\"\\([^\"]*\\)\"" lim t)
+	      (if (< 0 (length (match-string 1)))
+		  (setq consts (cons (match-string 1) consts))))))
+      consts)))
+
+
 (provide 'isabelle-system)
 ;; End of isabelle-system.el
