@@ -1,26 +1,26 @@
 ;; thy-mode.el - Mode for Isabelle theory files.
 ;;
 ;; Copyright (C) 1998 LFCS Edinburgh. 
-;; Author:    David Aspinall <da@dcs.ed.ac.uk>
+;; Author:    David Aspinall <David.Aspinall@ed.ac.uk>
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
 ;; Taken from Isamode, version: 3.6 1998/09/02 11:40:45
 ;;
-;; thy-mode.el,v 7.0 2002/08/29 09:14:04 da Exp
+;; thy-mode.el,v 8.0 2004/04/17 23:40:00 da Exp
 ;;
 ;; NAMESPACE management: all functions and variables declared
-;; in this file begin with isa-thy-
+;; in this file begin with thy-
 
 (require 'proof-site)
 (require 'proof-syntax)
 (require 'isa)
 
 ;; In case thy mode was invoked directly or by -*- thy -*- at
-;; the start of the file, ensure that Isar mode is used from now
+;; the start of the file, ensure that isa mode is used from now
 ;; on for .thy files.
 ;; FIXME: be less messy with auto-mode-alist here (remove dups)
 (setq auto-mode-alist
-      (cons '("\\.thy$" . thy-mode) auto-mode-alist))
+      (cons '("\\.thy$" . isa-mode) auto-mode-alist))
 
 ;;; ========== Theory File Mode User Options ==========
 
