@@ -1,11 +1,17 @@
-;; isar.el Major mode for Isabelle/Isar proof assistant
-;; Copyright (C) 1994-1998 LFCS Edinburgh.
+;; isar.el	  Major mode for Isabelle/Isar proof assistant
+;; Copyright (C) 1994-2004 LFCS Edinburgh.
+;;
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; Author:              David Aspinall <da@dcs.ed.ac.uk>
-;; Author / Maintainer: Markus Wenzel <wenzelm@in.tum.de>
+;; Maintainers:   David Aspinall, Stefan Berghofer
 ;;
-;; isar.el,v 7.1 2002/08/29 12:06:56 da Exp
+;; Authors:       David Aspinall <David.Aspinall@ed.ac.uk>
+;;   	          Markus Wenzel <wenzelm@in.tum.de>
+;;
+;; Contributors:  David von Oheimb, Sebastian Skalberg
+;;
+;;
+;; isar.el,v 8.0 2004/04/17 23:40:00 da Exp
 ;;
 
 (require 'proof)
@@ -116,11 +122,15 @@ See -k option for Isabelle interface script."
    proof-mode-for-script        'isar-mode
    ;; proof script syntax
    proof-terminal-char          ?\;     ; forcibly ends a command
-   proof-script-command-start-regexp (proof-regexp-alt
-                                      isar-any-command-regexp
-                                      (regexp-quote ";"))
+   proof-script-command-start-regexp 
+   (proof-regexp-alt
+    ;; FIXME: this gets { and } wrong: they must _not_ appear as {* *}
+    ;; because that's lexically a kind of comment.
+    isar-any-command-regexp
+    (regexp-quote ";"))
    proof-script-integral-proofs t
-;   proof-script-use-old-parser  t
+   ;; FIXME: use old parser for now to avoid { } problem
+   proof-script-use-old-parser  t
    proof-script-comment-start          isar-comment-start
    proof-script-comment-end            isar-comment-end
    proof-script-comment-start-regexp   isar-comment-start-regexp
@@ -132,9 +142,12 @@ See -k option for Isabelle interface script."
    ;; script buffer.
    proof-save-command-regexp    isar-save-command-regexp
    proof-goal-command-regexp    isar-goal-command-regexp
-   proof-goal-with-hole-regexp  isar-named-entity-regexp ; da
+   proof-goal-with-hole-regexp  isar-named-entity-regexp
+   proof-goal-with-hole-result	2
    proof-save-with-hole-regexp  nil
    proof-script-next-entity-regexps isar-next-entity-regexps
+   proof-script-imenu-generic-expression isar-generic-expression
+   imenu-syntax-alist isar-script-syntax-table-alist
 
    proof-indent-enclose-offset  (- proof-indent)
    proof-indent-open-offset     0
@@ -212,8 +225,14 @@ See -k option for Isabelle interface script."
    proof-shell-end-goals-regexp         "\367"
    pg-topterm-char			?\370
 
-   proof-assistant-setting-format       'isar-markup-ml
-   proof-shell-init-cmd                 (proof-assistant-settings-cmd)
+   ;; FIXME: next one is needed for backward compatibility.  
+   ;; Would be nice to remove this somehow else, it's only used for 
+   ;; Isar and pre-PGIP.  One way would be to hack the 
+   ;; (now obsolete) defpacustom calls.
+   proof-assistant-setting-format       
+   (unless isa-supports-pgip 'isar-markup-ml)
+
+   proof-shell-init-cmd			'(proof-assistant-settings-cmd)
    proof-shell-restart-cmd              "ProofGeneral.restart"
 
    proof-shell-eager-annotation-start-length 1
@@ -221,6 +240,11 @@ See -k option for Isabelle interface script."
    proof-shell-eager-annotation-end     "\361\\|\363"
    ;; see isar-pre-shell-start for proof-shell-trace-output-regexp
 
+   ;; Isabelle is learning to talk PGIP...
+   proof-shell-match-pgip-cmd		"<pgip"
+   proof-shell-issue-pgip-cmd		
+	(if isa-supports-pgip 'isabelle-process-pgip nil)
+
    ;; Some messages delimited by eager annotations
    proof-shell-clear-response-regexp    "Proof General, please clear the response buffer."
    proof-shell-clear-goals-regexp       "Proof General, please clear the goals buffer."
@@ -229,6 +253,10 @@ See -k option for Isabelle interface script."
    proof-shell-theorem-dependency-list-regexp "Proof General, theorem dependencies of \\(.*\\) are \"\\(.*\\)\"\361"
    proof-shell-theorem-dependency-list-split "\" \""
    proof-shell-show-dependency-cmd "thm %s;"
+   proof-shell-identifier-under-mouse-cmd 
+   '((nil     "thm %s;")
+     (string  "term \"%s\";")
+     (comment "term \"%s\";"))
 
    ;; Allow font-locking for output based on hidden annotations, see
    ;; isar-output-font-lock-keywords-1
@@ -246,6 +274,7 @@ See -k option for Isabelle interface script."
        'isabelle-convert-idmarkup-to-subterm 'pg-remove-specials)
    pg-subterm-help-cmd			"term %s" 
 
+   proof-cannot-reopen-processed-files  t
    proof-shell-process-file
    (cons
     ;; Theory loader output
@@ -295,22 +324,22 @@ proof-shell-retract-files-regexp."
 ;;
 (eval-and-compile
 (define-derived-mode isar-shell-mode proof-shell-mode
-   "Isabelle/Isar shell" nil
+   "Isar shell" nil
    (isar-shell-mode-config)))
 
 (eval-and-compile
 (define-derived-mode isar-response-mode proof-response-mode
-  "Isabelle/Isar response" nil
+  "response" nil
   (isar-response-mode-config)))
 
 (eval-and-compile                       ; to define vars for byte comp.
 (define-derived-mode isar-goals-mode proof-goals-mode
-  "Isabelle/Isar proofstate" nil
+  "proofstate" nil
   (isar-goals-mode-config)))
 
 (eval-and-compile                       ; to define vars for byte comp.
 (define-derived-mode isar-mode proof-mode
-  "Isabelle/Isar script" nil
+  "Isar script" nil
   (isar-mode-config)))
 
 
@@ -335,31 +364,35 @@ proof-shell-retract-files-regexp."
 (proof-definvisible isar-help-facts "print_facts" [h f])
 (proof-definvisible isar-help-syntax "print_syntax" [h i])
 (proof-definvisible isar-help-induct-rules "print_induct_rules" [h I])
+(proof-definvisible isar-help-intro-rules "print_intros" [h r])
 (proof-definvisible isar-help-methods "print_methods" [h m])
 (proof-definvisible isar-help-simpset "print_simpset" [h S])
 (proof-definvisible isar-help-binds "print_binds" [h b])
 (proof-definvisible isar-help-theorems "print_theorems" [h t])
 (proof-definvisible isar-help-trans-rules "print_trans_rules" [h T])
 
-(defpgdefault help-menu-entries
+(defpgdefault menu-entries
   (append
-   isabelle-docs-menu
+   (list isabelle-logics-menu)
    (list
     (cons "Show me ..."
           (list
+           ["cases"              isar-help-cases          t]
+           ["facts"              isar-help-facts          t]
+           ["matching rules"     isar-help-intro-rules    t]
+           ["term bindings"      isar-help-binds          t]
+	   "----"
+           ["classical rules"    isar-help-claset         t]
+           ["induct/cases rules" isar-help-induct-rules   t]
+           ["simplifier rules"   isar-help-simpset        t]
+           ["theorems"           isar-help-theorems       t]
+           ["transitivity rules" isar-help-trans-rules    t]
+	   "----"
            ["antiquotations"     isar-help-antiquotations t]
            ["attributes"         isar-help-attributes     t]
-           ["cases"              isar-help-cases          t]
-           ["classical rules"    isar-help-claset         t]
            ["commands"           isar-help-commands       t]
-           ["facts"              isar-help-facts          t]
            ["inner syntax"       isar-help-syntax         t]
-           ["induct/cases rules" isar-help-induct-rules   t]
-           ["methods"            isar-help-methods        t]
-           ["simplifier rules"   isar-help-simpset        t]
-           ["term bindings"      isar-help-binds          t]
-           ["theorems"           isar-help-theorems       t]
-           ["transitivity rules" isar-help-trans-rules    t])))))
+           ["methods"            isar-help-methods        t])))))
 
 ;; undo proof commands
 (defun isar-count-undos (span)
@@ -401,6 +434,7 @@ proof-shell-retract-files-regexp."
        ;;  str below)
        ;; FIXME: should adjust proof-nesting-depth here.
        ((or (eq (span-property span 'type) 'comment)
+	    (eq (span-property span 'type) 'proverproc)
 	    (eq (span-property span 'type) 'proof); da: needed?
             (proof-string-match isar-undo-skip-regexp str)
             (proof-string-match isar-undo-ignore-regexp str)))
@@ -410,7 +444,10 @@ proof-shell-retract-files-regexp."
        ;; open goal: skip and exit
        ((proof-string-match isar-goal-command-regexp str)
         (setq span nil))
-       ;; not undoable: fail and exit
+       ;; not undoable: fail and exit 
+       ;; [da: this is an odd case: it issues cannot_undo command to Isar,
+       ;;  which immediately generates an error, I think it's a bit confusing
+       ;; for the user]
        ((proof-string-match isar-undo-fail-regexp str)
         (setq answers nil)
         (setq ans (isar-cannot-undo str))
@@ -468,10 +505,12 @@ proof-shell-retract-files-regexp."
 ;;   Commands specific to isar                                      ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(proof-defshortcut isar-bold "\\<^bold>" [(control b)])
-(proof-defshortcut isar-super "\\<^sup>" [(control u)])
-(proof-defshortcut isar-sub "\\<^sub>"   [(control l)])
-
+(proof-defshortcut isar-super    "\\<^sup>" [(control u)])
+(proof-defshortcut isar-sub      "\\<^sub>"   [(control l)])
+(proof-defshortcut isar-longsup  "\\<^bsup>%p\\<^esup>" [u])
+(proof-defshortcut isar-longsub  "\\<^bsub>%p\\<^esub>" [l])
+(proof-defshortcut isar-idsub    "\\<^isub>" [(control i)])
+(proof-defshortcut isar-raw      "\\<^raw:%p>" [(control r)])
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -481,13 +520,13 @@ proof-shell-retract-files-regexp."
 ;borrowed from plastic.el
 (defvar isar-shell-current-line-width nil
   "Current line width of the Isabelle process's pretty printing module.
-  Its value will be updated whenever the corresponding screen gets
-  selected.")
+Its value will be updated whenever the corresponding screen gets
+selected.")
 
 ;borrowed from plastic.el
 (defun isar-shell-adjust-line-width ()
   "Use Isabelle's pretty printing facilities to adjust output line width.
-   Checks the width in the `proof-goals-buffer'"
+Checks the width in the `proof-goals-buffer'"
   (let ((ans ""))
     (and (buffer-live-p proof-goals-buffer)
          (proof-shell-live-buffer)
@@ -583,19 +622,15 @@ proof-shell-retract-files-regexp."
 	     x-symbol-isabelle-font-lock-keywords)))
   (proof-goals-config-done))
 
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
-;; x-symbol support
-;;
-;; The following settings configure the generic PG package.
-;; The token language "Isabelle Symbols" is in file isa/x-symbol-isabelle.el
+;; Remove isa-mode from auto-mode-alist, 
+;; to allow SML mode to work in preference to isa-mode.
 ;;
 
-(setq
- proof-xsym-activate-command
- (isar-markup-ml "print_mode := ([\"xsymbols\", \"symbols\"] @ ! print_mode)")
- proof-xsym-deactivate-command
- (isar-markup-ml "print_mode := (Library.gen_rems (op =) (! print_mode, [\"xsymbols\", \"symbols\"]))"))
+(setq auto-mode-alist
+      (delete-if 
+       (lambda (strmod) (memq (cdr strmod) '(isa-mode demoisa-mode)))
+       auto-mode-alist))
 
 (provide 'isar)
